@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -53,7 +53,7 @@ static Boolean play_moves(Game *game_details, Board *board, Move *moves,
 static Boolean apply_variations(const Game *game_details,const Board *board,
                                 Variation *variation, Boolean check_move_validity);
 static Boolean rewrite_variations(const Board *board,Variation *variation, Boolean null_move_found);
-static Boolean rewrite_moves(Board *board,Move *moves, Boolean null_move_found);
+static Boolean rewrite_moves(Board *board, Move *move_details, Boolean null_move_found);
 static void append_evaluation(Move *move_details, const Board *board);
 static void append_FEN_comment(Move *move_details, const Board *board);
 static double evaluate(const Board *board);
@@ -103,7 +103,7 @@ void set_output_piece_characters(const char *letters)
                 piece_index++;
                 if(letters[piece_index] != '\0'){
                     double_char_piece[1] = letters[piece_index];
-                    output_piece_characters[piece] = StringCopy(double_char_piece);
+                    output_piece_characters[piece] = copy_string(double_char_piece);
                     piece_index++;
                 }
                 else{
@@ -116,7 +116,7 @@ void set_output_piece_characters(const char *letters)
             else{
                 static char single_char_piece[] = "X";
                 *single_char_piece = letters[piece_index];
-                output_piece_characters[piece] = StringCopy(single_char_piece);
+                output_piece_characters[piece] = copy_string(single_char_piece);
                 piece_index++;
             }
         }
@@ -144,8 +144,9 @@ void set_output_piece_characters(const char *letters)
     }
 }
 
+      /* Return a fresh copy of the given string. */
 char *
-StringCopy(const char *str)
+copy_string(const char *str)
 {   char *result;
     size_t len = strlen(str);
 
@@ -199,7 +200,7 @@ is_FEN_piece(char c)
 char
 SAN_piece_letter(Piece piece)
 {
-    if(piece >= 0 && piece < NUM_PIECE_VALUES){
+    if(piece < NUM_PIECE_VALUES){
         return SAN_piece_characters[piece];
     }
     else{
@@ -588,7 +589,6 @@ new_game_board(const char *fen)
             }
         }
     }
-
     return new_board;
 }
 
@@ -716,7 +716,7 @@ apply_move(Colour colour,Move *move_details, Board *board)
         }
 
 	if(move_details->class != NULL_MOVE) {
-	    make_move(move_details->from_col,move_details->from_rank,
+	    make_move(move_details->class, move_details->from_col,move_details->from_rank,
 			    move_details->to_col,move_details->to_rank,
 			    piece_to_move,colour,board);
 	}
@@ -728,29 +728,27 @@ apply_move(Colour colour,Move *move_details, Board *board)
                 /* Nothing more to do. */
                 break;
             case PAWN_MOVE_WITH_PROMOTION:
-                if(move_details->class == PAWN_MOVE_WITH_PROMOTION){
-                    if(move_details->promoted_piece != EMPTY){
-                        /* Now make the promotion. */
-                        make_move(move_details->to_col,move_details->to_rank,
-                                  move_details->to_col,move_details->to_rank,
-                                  move_details->promoted_piece,colour,board);
-                    }
-                    else{
-                        Ok = FALSE;
-                    }
+                if(move_details->promoted_piece != EMPTY){
+                    /* Now make the promotion. */
+                    make_move(move_details->class, move_details->to_col,move_details->to_rank,
+                              move_details->to_col,move_details->to_rank,
+                              move_details->promoted_piece,colour,board);
+                }
+                else{
+                    Ok = FALSE;
                 }
                 break;
         case KINGSIDE_CASTLE:
             /* Step the Rook to the left of the king. */
             piece_to_move = ROOK;
-            make_move(LASTCOL,move_details->from_rank,move_details->to_col-1,
+            make_move(move_details->class, LASTCOL,move_details->from_rank,move_details->to_col-1,
                             move_details->to_rank,
                             piece_to_move,colour,board);
             break;
         case QUEENSIDE_CASTLE:
             /* Step the Rook to the right of the king. */
             piece_to_move = ROOK;
-            make_move(FIRSTCOL,move_details->from_rank,move_details->to_col+1,
+            make_move(move_details->class, FIRSTCOL,move_details->from_rank,move_details->to_col+1,
                             move_details->to_rank,
                             piece_to_move,colour,board);
             break;
@@ -772,14 +770,6 @@ apply_move(Colour colour,Move *move_details, Board *board)
                     move_details->check_status = CHECKMATE;
                 }
             }
-            /* Handle the halfmove_clock. */
-            if(piece_to_move == PAWN ||
-                    move_details->captured_piece != EMPTY) {
-                board->halfmove_clock = 0;
-            }
-            else {
-                board->halfmove_clock++;
-            }
         }
     }
     else{
@@ -798,11 +788,15 @@ static Boolean
 play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
            Boolean check_move_validity)
 {   Boolean game_ok = TRUE;
+    /* Ply number at which any error was found. */
+    int error_ply = 0;
     /* Force a match if we aren't looking for positional variations. */
     Boolean game_matches = GlobalState.positional_variations?FALSE:TRUE;
     Move *next_move = moves;
     /* Keep track of the final ECO match. */
     EcoLog *eco_match = NULL;
+    /* Whether a NULL_MOVE was found in the main line. */
+    Boolean null_move_in_main_line = FALSE;
 
     /* Try the initial board position for a match.
      * This is required because the game might have been set up
@@ -832,6 +826,7 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
             }
             /* Now try the main move. */
 	    if(next_move->class == NULL_MOVE) {
+		null_move_in_main_line = TRUE;
 	        /* We might not be able to check the validity of
 		 * subsequent moves.
 		 */
@@ -911,6 +906,12 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
 		    print_board(board,GlobalState.logfile);
 		    report_details(GlobalState.logfile);
 		    game_ok = FALSE;
+		    /* Work out where the error was. */
+		    error_ply = 2 * board->move_number - 1;
+		    /* Check who has just moved. */
+		    if(board->to_move == BLACK) {
+			error_ply++;
+		    }
 		}
             }
 	    else {
@@ -928,8 +929,17 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
                         "Internal error: Empty move in play_moves.\n");
             report_details(GlobalState.logfile);
             game_ok = FALSE;
+	    /* Work out where the error was. */
+	    error_ply = 2 * board->move_number - 1;
+	    /* Check who has just moved. */
+	    if(board->to_move == BLACK) {
+		error_ply++;
+	    }
         }
     }
+    if(null_move_in_main_line) {
+        game_ok = FALSE;
+    }
     if(game_ok){
         if(eco_match != NULL){
            /* Free any details of the old one. */
@@ -951,29 +961,35 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
            }
            /* Add in the new one. */
            if(eco_match->ECO_tag != NULL){
-               game_details->tags[ECO_TAG] = StringCopy(eco_match->ECO_tag);
+               game_details->tags[ECO_TAG] = copy_string(eco_match->ECO_tag);
            }
            if(eco_match->Opening_tag != NULL){
-               game_details->tags[OPENING_TAG] = StringCopy(eco_match->Opening_tag);
+               game_details->tags[OPENING_TAG] = copy_string(eco_match->Opening_tag);
            }
            if(eco_match->Variation_tag != NULL){
                game_details->tags[VARIATION_TAG] =
-                                StringCopy(eco_match->Variation_tag);
+                                copy_string(eco_match->Variation_tag);
            }
            if(eco_match->Sub_Variation_tag != NULL){
                game_details->tags[SUB_VARIATION_TAG] =
-                                StringCopy(eco_match->Sub_Variation_tag);
+                                copy_string(eco_match->Sub_Variation_tag);
            }
         }
     }
     /* Fill in the hash value of the final position reached. */
     game_details->final_hash_value = board->hash_value;
     game_details->moves_ok = game_ok;
+    game_details->error_ply = error_ply;
     if(!game_ok){
-        /* Only return a match if it genuinely matched a variation
-         * in which we were interested.
-         */
-        if(!GlobalState.positional_variations){
+	/* Decide whether to keep it anyway. */
+	if(GlobalState.keep_broken_games) {
+	}
+	else if(GlobalState.positional_variations){
+	}
+	else {
+	    /* Only return a match if it genuinely matched a variation
+	     * in which we were interested.
+	     */
             /* We can't have found a genuine match. */
             game_matches = FALSE;
         }
@@ -989,6 +1005,8 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
 static void
 play_eco_moves(Game *game_details, Board *board,Move *moves)
 {   Boolean game_ok = TRUE;
+    /* Ply number at which any error was found. */
+    int error_ply = 0;
     Move *next_move = moves;
 
     /* Keep going while the game is ok, and we have some more
@@ -1017,6 +1035,12 @@ play_eco_moves(Game *game_details, Board *board,Move *moves)
                 print_board(board,GlobalState.logfile);
                 report_details(GlobalState.logfile);
                 game_ok = FALSE;
+		/* Work out where the error was. */
+		error_ply = 2 * board->move_number - 1;
+		/* Check who has just moved. */
+		if(board->to_move == BLACK) {
+		    error_ply++;
+		}
             }
         }
         else{
@@ -1025,11 +1049,18 @@ play_eco_moves(Game *game_details, Board *board,Move *moves)
                         "Internal error: Empty move in play_eco_moves.\n");
             report_details(GlobalState.logfile);
             game_ok = FALSE;
+	    /* Work out where the error was. */
+	    error_ply = 2 * board->move_number - 1;
+	    /* Check who has just moved. */
+	    if(board->to_move == BLACK) {
+		error_ply++;
+	    }
         }
     }
     /* Fill in the hash value of the final position reached. */
     game_details->final_hash_value = board->hash_value;
     game_details->moves_ok = game_ok;
+    game_details->error_ply = error_ply;
 }
 
         /* Play out a variation.
@@ -1143,7 +1174,7 @@ apply_eco_move_list(Game *game_details,unsigned *number_of_half_moves)
 const char *
 piece_str(Piece piece)
 {
-    if(piece >= 0 && piece < NUM_PIECE_VALUES){
+    if(piece < NUM_PIECE_VALUES){
         return output_piece_characters[piece];
     }
     else{
@@ -1377,7 +1408,7 @@ rewrite_move(Colour colour,Move *move_details, Board *board, Boolean null_move_f
         Piece piece_to_move = move_details->piece_to_move;
 
 	if(move_details->class != NULL_MOVE) {
-	    make_move(move_details->from_col,move_details->from_rank,
+	    make_move(move_details->class, move_details->from_col,move_details->from_rank,
 			    move_details->to_col,move_details->to_rank,
 			    piece_to_move,colour,board);
 	}
@@ -1396,7 +1427,7 @@ rewrite_move(Colour colour,Move *move_details, Board *board, Boolean null_move_f
                     if(move_details->promoted_piece != EMPTY){
                         /* @@@ Do promoted moves have '+' properly appended? */
                         /* Now make the promotion. */
-                        make_move(move_details->to_col,move_details->to_rank,
+                        make_move(move_details->class, move_details->to_col,move_details->to_rank,
                                   move_details->to_col,move_details->to_rank,
                                   move_details->promoted_piece,colour,board);
                     }
@@ -1408,14 +1439,14 @@ rewrite_move(Colour colour,Move *move_details, Board *board, Boolean null_move_f
         case KINGSIDE_CASTLE:
             /* Step the Rook to the left of the king. */
             piece_to_move = ROOK;
-            make_move(LASTCOL,move_details->from_rank,move_details->to_col-1,
+            make_move(move_details->class, LASTCOL,move_details->from_rank,move_details->to_col-1,
                             move_details->to_rank,
                             piece_to_move,colour,board);
             break;
         case QUEENSIDE_CASTLE:
             /* Step the Rook to the right of the king. */
             piece_to_move = ROOK;
-            make_move(FIRSTCOL,move_details->from_rank,move_details->to_col+1,
+            make_move(move_details->class, FIRSTCOL,move_details->from_rank,move_details->to_col+1,
                             move_details->to_rank,
                             piece_to_move,colour,board);
             break;
@@ -1436,7 +1467,7 @@ rewrite_move(Colour colour,Move *move_details, Board *board, Boolean null_move_f
 
         /* Rewrite the list of moves by playing through the game. */
 static Boolean
-rewrite_moves(Board *board,Move *moves, Boolean null_move_found)
+rewrite_moves(Board *board, Move *moves, Boolean null_move_found)
 {   Boolean game_ok = TRUE;
     Move *move_details = moves;
 
@@ -1455,6 +1486,9 @@ rewrite_moves(Board *board,Move *moves, Boolean null_move_found)
 	    }
             if(rewrite_move(board->to_move,move_details,board, null_move_found)){
                 board->to_move = OPPOSITE_COLOUR(board->to_move);
+                if(board->to_move == WHITE){
+                    board->move_number++;
+                }
 
                 if(GlobalState.output_evaluation) {
                     /* Append an evaluation of the new state of the board
@@ -1464,25 +1498,27 @@ rewrite_moves(Board *board,Move *moves, Boolean null_move_found)
                 }
 
                 if(GlobalState.add_FEN_comments) {
-                    /* Append an FEN comment with the new state of the board
+                    /* Append a FEN comment with the new state of the board
                      * with the move having been played.
                      */
                     append_FEN_comment(move_details, board);
                 }
 
                 move_details = move_details->next;
-                if(board->to_move == WHITE){
-                    board->move_number++;
-                }
             }
             else {
-                fprintf(GlobalState.logfile,
-                                "Failed to rewrite move %u%s %s in the game:\n",
-                                    board->move_number,
-                                    (board->to_move == WHITE)?".":"...",
-                                    move_details->move);
-                report_details(GlobalState.logfile);
-                print_board(board,GlobalState.logfile);
+		/* Broken games that are being kept have probably already been
+		 * reported, so don't repeat the error.
+		 */
+		if(!GlobalState.keep_broken_games) {
+		    fprintf(GlobalState.logfile,
+				    "Failed to rewrite move %u%s %s in the game:\n",
+					board->move_number,
+					(board->to_move == WHITE)?".":"...",
+					move_details->move);
+		    report_details(GlobalState.logfile);
+		    print_board(board,GlobalState.logfile);
+		}
                 game_ok = FALSE;
             }
         }
@@ -1494,6 +1530,40 @@ rewrite_moves(Board *board,Move *moves, Boolean null_move_found)
             game_ok = FALSE;
         }
     }
+    if(!game_ok && GlobalState.keep_broken_games && move_details != NULL) {
+        /* Try to place the remaining moves into a comment. */
+	CommentList *comment = (CommentList* ) MallocOrDie(sizeof(*comment));
+	/* Break the link from the previous move. */
+	Move *prev;
+	StringList *commented_move_list = NULL;
+
+	/* Find the move before the current one. */
+	prev = moves;
+	while(prev != NULL && prev->next != move_details) {
+	    prev = prev->next;
+	}
+	if(prev != NULL) {
+	    prev->next = NULL;
+	}
+	while(move_details != NULL) {
+	    commented_move_list = save_string_list_item(commented_move_list, (char *) move_details->move);
+	    if(move_details->next == NULL && prev != NULL) {
+	        /* Pass the terminating result to prev. */
+		prev->terminating_result = move_details->terminating_result;
+		move_details->terminating_result = NULL;
+		move_details = NULL;
+	    }
+	    else {
+		move_details = move_details->next;
+	    }
+	}
+	comment->Comment = commented_move_list;
+	comment->next = NULL;
+
+	if(prev != NULL) {
+	    append_comments_to_move(prev, comment);
+	}
+    }
     return game_ok;
 }
 
@@ -1522,13 +1592,19 @@ rewrite_variations(const Board *board,Variation *variation, Boolean null_move_fo
          * NULL otherwise.
          */
 Board *
-rewrite_game(Move *moves,const char *fen)
-{   Board *board = new_game_board(fen);
+rewrite_game(Game *current_game)
+{
+    const char *fen = current_game->tags[FEN_TAG];
+    Board *board = new_game_board(fen);
     Boolean game_ok;
 
     /* No null-move found at the start of the game. */
-    game_ok = rewrite_moves(board,moves,FALSE);
-    if(!game_ok){
+    game_ok = rewrite_moves(board,current_game->moves,FALSE);
+    if(game_ok){
+    }
+    else if(GlobalState.keep_broken_games) {
+    }
+    else {
         (void) free_board((void *)board);
         board = NULL;
     }
@@ -1681,8 +1757,55 @@ build_basic_EPD_string(const Board *board,char *epd)
 
     /* Enpassant. */
     if(board->EnPassant){
-        epd[ix] = board->ep_col; ix++;
-        epd[ix] = board->ep_rank; ix++;
+        /* It might be required to suppress redundant ep info. */
+        Boolean suppress = FALSE;
+        if(GlobalState.suppress_redundant_ep_info) {
+            /* Determine whether the ep indication is redundant or not.
+             * Assume that it is unless there is a pawn in position to
+             * take advantage of it.
+             */
+            Boolean redundant = TRUE;
+            Col ep_col = board->ep_col;
+            Rank from_rank;
+            Piece pawn;
+            if (board->to_move == WHITE) {
+                /* White pawn on the fifth rank capturing a black pawn. */
+                from_rank = '5';
+                pawn = W(PAWN);
+            }
+            else {
+                /* Black pawn on the fourth rank capturing a white pawn. */
+                from_rank = '4';
+                pawn = B(PAWN);
+            }
+            if ((ep_col > FIRSTCOL) && (board->board[RankConvert(from_rank)][ColConvert(ep_col - 1)] == pawn)) {
+                /* Check that the move does not leave the king in check. */
+                Board copy_board = *board;
+                make_move(UNKNOWN_MOVE, ep_col - 1, from_rank,
+                          board->ep_col, board->ep_rank, PAWN, board->to_move, &copy_board);
+                if(king_is_in_check(&copy_board, copy_board.to_move) == NOCHECK){
+                    redundant = FALSE;
+                }
+            }
+            if (redundant && (ep_col < LASTCOL) &&
+                  (board->board[RankConvert(from_rank)][ColConvert(ep_col + 1)] == pawn)) {
+                /* Check that the move does not leave the king in check. */
+                Board copy_board = *board;
+                make_move(UNKNOWN_MOVE, ep_col + 1,from_rank,
+                          board->ep_col,board->ep_rank,PAWN,board->to_move,&copy_board);
+                if(king_is_in_check(&copy_board, copy_board.to_move) == NOCHECK){
+                    redundant = FALSE;
+                }
+            }
+            suppress = redundant;
+        }
+        if(suppress) {
+          epd[ix] = '-'; ix++;
+        }
+        else {
+          epd[ix] = board->ep_col; ix++;
+          epd[ix] = board->ep_rank; ix++;
+        }
     }
     else{
         epd[ix] = '-'; ix++;
@@ -1695,8 +1818,6 @@ void
 build_FEN_string(const Board *board,char *fen)
 {   
     size_t ix;
-    int full_move_number =
-            board->to_move == BLACK ? board->move_number : (board->move_number + 1);
 
     build_basic_EPD_string(board,fen);
     /* Append the (pseudo) half move count and the full move count. */
@@ -1709,7 +1830,7 @@ build_FEN_string(const Board *board,char *fen)
     fen[ix] = ' '; ix++;
 
     /* The full move number. */
-    sprintf(&fen[ix],"%u", full_move_number);
+    sprintf(&fen[ix],"%u", board->move_number);
 }
 
     /* Append to move_details a FEN comment of the board.
@@ -1721,7 +1842,7 @@ append_FEN_comment(Move *move_details, const Board *board)
 {
     char *FEN_comment = MallocOrDie(FEN_SPACE);
     CommentList *comment = (CommentList* ) MallocOrDie(sizeof(*comment));
-    StringList *current_comment = SaveStringListItem(NULL, FEN_comment);
+    StringList *current_comment = save_string_list_item(NULL, FEN_comment);
     
     build_FEN_string(board, FEN_comment);
     comment->Comment = current_comment;
@@ -1755,7 +1876,7 @@ append_evaluation(Move *move_details, const Board *board)
         exit(1);
     }
 
-    current_comment = SaveStringListItem(NULL, evaluation);
+    current_comment = save_string_list_item(NULL, evaluation);
     comment->Comment = current_comment;
     comment->next = NULL;
     append_comments_to_move(move_details, comment);
@@ -1768,8 +1889,8 @@ CommentList *
 create_match_comment(Move *move_details)
 {
     /* The comment string. */
-    char *match_comment = StringCopy(GlobalState.position_match_comment);
-    StringList *current_comment = SaveStringListItem(NULL, match_comment);
+    char *match_comment = copy_string(GlobalState.position_match_comment);
+    StringList *current_comment = save_string_list_item(NULL, match_comment);
     CommentList *comment = (CommentList* ) MallocOrDie(sizeof(*comment));
     
     comment->Comment = current_comment;
