@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -31,6 +31,7 @@
 #include "taglist.h"
 #include "tokens.h"
 #include "lex.h"
+#include "taglines.h"
 #include "moves.h"
 #include "end.h"
 #include "eco.h"
@@ -40,7 +41,7 @@
 #include "lists.h"
 #include "mymalloc.h"
 
-#define CURRENT_VERSION "v17-14"
+#define CURRENT_VERSION "v17-21"
 #define URL "http://www.cs.kent.ac.uk/people/staff/djb/pgn-extract/"
 
 /* The prefix of the arguments allowed in an argsfile.
@@ -153,7 +154,7 @@ usage_and_exit(void)
       "-vvariations -- the file variations contains the textual lines of interest.",
       "-V -- don't include variations in the output. Ordinarily these are retained.",
       "-wwidth -- set width as an approximate line width for output.",
-      "-W[cm|epd|halg|lalg|elalg|san] -- specify the output format to use.",
+      "-W[cm|epd|halg|lalg|elalg|xlalg|san] -- specify the output format to use.",
       "      Default is SAN.",
       "      -W means use the input format.",
       "      -Wcm is (a possibly obsolete) ChessMaster format.",
@@ -162,6 +163,7 @@ usage_and_exit(void)
       "      -Whalg is hyphenated long algebraic.",
       "      -Wlalg is long algebraic.",
       "      -Welalg is enhanced long algebraic.",
+      "      -Wxlalg is enhanced long algebraic with x for captures and - for non capture moves.",
       "      -Wuci is output compatible with the UCI protocol.",
       "-xvariations -- the file variations contains the lines resulting in",
       "                positions of interest.",
@@ -171,6 +173,7 @@ usage_and_exit(void)
 
       "",
 
+      "--addhashcode - output a HashCode tag",
       "--append - see -a",
       "--checkfile - see -c",
       "--checkmate - see -M",
@@ -179,11 +182,13 @@ usage_and_exit(void)
       "--fencomments - include a FEN string after each move",
       "--fuzzydepth plies - positional duplicates match",
       "--help - see -h",
+      "--keepbroken - retain games with errors",
       "--linelength - see -w",
       "--markmatches - mark positional and material matches with a comment; see -t, -v, and -z",
       "--nochecks - don't output + and # after moves.",
       "--nocomments - see -C",
       "--noduplicates - see -D",
+      "--nofauxep - don't output ep squares in FEN when the capture is not possible",
       "--nomovenumbers - don't output move numbers.",
       "--nonags - see -N",
       "--noresults - don't output results.",
@@ -192,8 +197,10 @@ usage_and_exit(void)
       "--novars - see -V",
       "--output - see -o",
       "--plylimit - limit the number of plies output.",
+      "--selectonly N - only output the Nth matched game (N > 0)",
       "--seven - see -7",
       "--stalemate - only output games that end in stalemate.",
+      "--totalplycount - include a tag with the total number of plies in a game.",
       "--version - print the current version number and exit.",
 
       /* Must be NULL terminated. */
@@ -206,7 +213,7 @@ usage_and_exit(void)
           "pgn-extract %s (%s): a Portable Game Notation (PGN) manipulator.\n",
           CURRENT_VERSION,__DATE__);
   fprintf(GlobalState.logfile,
-          "Copyright (C) 1994-2013 David J. Barnes (d.j.barnes@kent.ac.uk)\n");
+          "Copyright (C) 1994-2015 David J. Barnes (d.j.barnes@kent.ac.uk)\n");
   fprintf(GlobalState.logfile,"%s\n\n",URL);
   fprintf(GlobalState.logfile,"Usage: pgn-extract [arguments] [file.pgn ...]\n");
   
@@ -486,10 +493,10 @@ process_argument(char arg_letter,const char *associated_value)
                   (void) fclose(GlobalState.outputfile);
               }
               if(arg_letter == WRITE_TO_OUTPUT_FILE_ARGUMENT){
-                  GlobalState.outputfile = MustOpen(filename,"w");
+                  GlobalState.outputfile = must_open_file(filename,"w");
               }
               else{
-                  GlobalState.outputfile = MustOpen(filename,"a");
+                  GlobalState.outputfile = must_open_file(filename,"a");
               }
               GlobalState.output_filename = filename;
           }
@@ -523,13 +530,13 @@ process_argument(char arg_letter,const char *associated_value)
               exit(1);
           }
           else{
-              GlobalState.duplicate_file = MustOpen(filename,"w");
+              GlobalState.duplicate_file = must_open_file(filename,"w");
           }
           break;
         case USE_ECO_FILE_ARGUMENT:
           GlobalState.add_ECO = TRUE;
           if(*filename != '\0'){
-              GlobalState.eco_file = StringCopy(filename);
+              GlobalState.eco_file = copy_string(filename);
           }
           else if((filename = getenv("ECO_FILE")) != NULL){
               GlobalState.eco_file = filename;
@@ -587,7 +594,7 @@ process_argument(char arg_letter,const char *associated_value)
                   add_filename_to_source_list(filename,CHECKFILE);
               }
               else{
-                  FILE *fp = MustOpen(filename,"r");
+                  FILE *fp = must_open_file(filename,"r");
                   add_filename_list_from_file(fp,CHECKFILE);
                   (void) fclose(fp);
               }
@@ -595,7 +602,7 @@ process_argument(char arg_letter,const char *associated_value)
           break;
         case FILE_OF_FILES_ARGUMENT:
           if(*filename != '\0'){
-              FILE *fp = MustOpen(filename,"r");
+              FILE *fp = must_open_file(filename,"r");
               add_filename_list_from_file(fp,NORMALFILE);
               (void) fclose(fp);
           }
@@ -714,7 +721,7 @@ process_argument(char arg_letter,const char *associated_value)
               if(GlobalState.non_matching_file != NULL){
                   (void) fclose(GlobalState.non_matching_file);
               }
-              GlobalState.non_matching_file = MustOpen(filename,"w");
+              GlobalState.non_matching_file = must_open_file(filename,"w");
           }
           else{
               fprintf(GlobalState.logfile,"Usage: -%cfilename.\n",arg_letter);
@@ -851,7 +858,7 @@ process_argument(char arg_letter,const char *associated_value)
         case MOVES_ARGUMENT:
             if(*filename != '\0'){
                 /* Where the list of variations of interest are kept. */
-                FILE *variation_file = MustOpen(filename,"r");
+                FILE *variation_file = must_open_file(filename,"r");
                 /* We wish to search for particular variations. */
                 add_textual_variations_from_file(variation_file);
                 fclose(variation_file);
@@ -859,7 +866,7 @@ process_argument(char arg_letter,const char *associated_value)
             break;
         case POSITIONS_ARGUMENT:
             if(*filename != '\0'){
-                FILE *variation_file = MustOpen(filename,"r");
+                FILE *variation_file = must_open_file(filename,"r");
                 /* We wish to search for positional variations. */
                 add_positional_variations_from_file(variation_file);
                 fclose(variation_file);
@@ -887,7 +894,11 @@ process_argument(char arg_letter,const char *associated_value)
 int
 process_long_form_argument(const char *argument, const char *associated_value)
 {
-    if(stringcompare(argument, "append") == 0) {
+    if(stringcompare(argument, "addhashcode") == 0) {
+        GlobalState.add_hashcode_tag = TRUE;
+        return 1;
+    }
+    else if(stringcompare(argument, "append") == 0) {
         process_argument(APPEND_TO_OUTPUT_FILE_ARGUMENT, associated_value);
         return 2;
     }
@@ -915,10 +926,6 @@ process_long_form_argument(const char *argument, const char *associated_value)
 	GlobalState.output_FEN_string = FALSE;
         return 1;
     }
-    else if(stringcompare(argument, "help") == 0) {
-        process_argument(HELP_ARGUMENT, "");
-        return 1;
-    }
     else if(stringcompare(argument, "fuzzydepth") == 0) {
         /* Extract the depth. */
         int depth = 0;
@@ -941,6 +948,14 @@ process_long_form_argument(const char *argument, const char *associated_value)
         }
         return 2;
     }
+    else if(stringcompare(argument, "help") == 0) {
+        process_argument(HELP_ARGUMENT, "");
+        return 1;
+    }
+    else if(stringcompare(argument, "keepbroken") == 0) {
+        GlobalState.keep_broken_games = TRUE;
+        return 1;
+    }
     else if(stringcompare(argument, "linelength") == 0) {
         process_argument(LINE_WIDTH_ARGUMENT,
                          associated_value);
@@ -949,7 +964,7 @@ process_long_form_argument(const char *argument, const char *associated_value)
     else if(stringcompare(argument, "markmatches") == 0) {
         if(*associated_value != '\0') {
 	    GlobalState.add_position_match_comments = TRUE;
-	    GlobalState.position_match_comment = StringCopy(associated_value);
+	    GlobalState.position_match_comment = copy_string(associated_value);
 	}
 	else {
               fprintf(GlobalState.logfile,
@@ -970,6 +985,10 @@ process_long_form_argument(const char *argument, const char *associated_value)
         process_argument(DONT_KEEP_DUPLICATES_ARGUMENT, "");
         return 1;
     }
+    else if(stringcompare(argument, "nofauxep") == 0) {
+        GlobalState.suppress_redundant_ep_info = TRUE;
+        return 1;
+    }
     else if(stringcompare(argument, "nomovenumbers") == 0) {
         GlobalState.keep_move_numbers = FALSE;
         return 1;
@@ -1002,6 +1021,27 @@ process_long_form_argument(const char *argument, const char *associated_value)
         process_argument(DONT_KEEP_VARIATIONS_ARGUMENT, "");
         return 1;
     }
+    else if(stringcompare(argument, "selectonly") == 0) {
+          unsigned long selection = 0;
+
+          /* Extract the selected match number. */
+          if(sscanf(associated_value, "%lu",&selection) == 1){
+              if(selection > 0) {
+                  GlobalState.matching_game_number = selection;
+              }
+              else {
+                  fprintf(GlobalState.logfile,
+                        "--%s requires a number greater than zero.\n", argument);
+                  exit(1);
+              }
+          }
+          else {
+              fprintf(GlobalState.logfile,
+                        "--%s requires a number greater than zero following it.\n", argument);
+              exit(1);
+          }
+          return 2;
+    }
     else if(stringcompare(argument, "output") == 0) {
         process_argument(WRITE_TO_OUTPUT_FILE_ARGUMENT, associated_value);
         return 2;
@@ -1016,13 +1056,13 @@ process_long_form_argument(const char *argument, const char *associated_value)
               }
               else {
                   fprintf(GlobalState.logfile,
-                        "--plylimit requires a number greater than or equal to zero.\n");
+                        "--%s requires a number greater than or equal to zero.\n", argument);
                   exit(1);
               }
           }
           else {
               fprintf(GlobalState.logfile,
-                        "--plylimit requires a number following it.\n");
+                        "--%s requires a number following it.\n", argument);
               exit(1);
           }
           return 2;
@@ -1035,6 +1075,10 @@ process_long_form_argument(const char *argument, const char *associated_value)
         GlobalState.match_only_stalemate = TRUE;
         return 1;
     }
+    else if(stringcompare(argument, "totalplycount") == 0) {
+        GlobalState.output_total_plycount = TRUE;
+        return 1;
+    }
     else if(stringcompare(argument, "version") == 0) {
         fprintf(GlobalState.logfile, "pgn-extract %s\n", CURRENT_VERSION);
 	exit(0);
