@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -30,7 +30,7 @@ typedef struct EcoLog {
     /* How deep the line is, from the half_moves associated with
      * the board when the line is played out.
      */
-    int half_moves;
+    unsigned half_moves;
     const char *ECO_tag;
     const char *Opening_tag;
     const char *Variation_tag;
