@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -497,10 +497,8 @@ piece_set_match(Ending_details *details_to_find,
          */
 static Boolean
 ending_match(Ending_details *details_to_find, int num_pieces[2][NUM_PIECE_VALUES],
-        Boolean white_first)
+        Colour game_colour)
 {   Boolean match = TRUE;
-    /* Select which way round to try the match. */
-    Colour game_colour = white_first? WHITE : BLACK;
     Colour piece_set_colour = WHITE;
 
     match = piece_set_match(details_to_find,num_pieces,game_colour,
