@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -127,8 +127,8 @@ add_fen_pattern(const char *fen_pattern)
         ok = FALSE;
     }
     if(ok) {
-	const char *pattern = StringCopy(fen_pattern);
-	fen_patterns = SaveStringListItem(fen_patterns, pattern);
+	const char *pattern = copy_string(fen_pattern);
+	fen_patterns = save_string_list_item(fen_patterns, pattern);
     }
     else {
         fprintf(GlobalState.logfile, "FEN Pattern: %s badly formed.\n",
