@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -40,8 +40,8 @@
 #include "grammar.h"
 #include "hashing.h"
 
-static TokenType Symbol = NO_TOKEN;
-int yyparse(SourceFileType file_type);
+static TokenType current_symbol = NO_TOKEN;
+
 /* Keep track of which RAV level we are at.
  * This is used to check whether a TERMINATING_RESULT is the final one
  * and whether NULL_MOVEs are allowed.
@@ -55,9 +55,9 @@ static unsigned RAV_level = 0;
 static struct{
     /* The tag values. */
     char **Tags;
-    int header_tags_length;
+    unsigned header_tags_length;
     CommentList *prefix_comment;
-} Game_Header;
+} GameHeader;
 
 static void ParseOptGameList(SourceFileType file_type);
 static Boolean ParseGame(Move **returned_move_list);
@@ -79,6 +79,8 @@ void free_tags(void);
 static void check_result(char **Tags,const char *terminating_result);
 static void free_comment_list(CommentList *comment_list);
 static void DealWithEcoLine(Move *move_list);
+static void DealWithGame(Move *move_list);
+static Boolean finished_processing(void);
 
     /* Initialise the game header structure to contain
      * space for the default number of tags.
@@ -88,36 +90,37 @@ static void DealWithEcoLine(Move *move_list);
 void
 init_game_header(void)
 {
-    int i;
-    Game_Header.header_tags_length = ORIGINAL_NUMBER_OF_TAGS;
-    Game_Header.Tags = (char **) MallocOrDie(Game_Header.header_tags_length*
-                    sizeof(*Game_Header.Tags));
-    for(i = 0; i < Game_Header.header_tags_length; i++){
-        Game_Header.Tags[i] = (char *) NULL;
+    unsigned i;
+    GameHeader.header_tags_length = ORIGINAL_NUMBER_OF_TAGS;
+    GameHeader.Tags = (char **) MallocOrDie(GameHeader.header_tags_length*
+                    sizeof(*GameHeader.Tags));
+    for(i = 0; i < GameHeader.header_tags_length; i++){
+        GameHeader.Tags[i] = (char *) NULL;
     }
 }
 
-void increase_game_header_tags_length(int new_length)
-{   int i;
+void
+increase_game_header_tags_length(unsigned new_length)
+{   unsigned i;
 
-    if(new_length < 0 || new_length <= Game_Header.header_tags_length){
+    if(new_length <= GameHeader.header_tags_length){
         fprintf(GlobalState.logfile,
                 "Internal error: inappropriate length %d ",new_length);
         fprintf(GlobalState.logfile,
                 " passed to increase_game_header_tags().\n");
         exit(1);
     }
-    Game_Header.Tags = (char **) ReallocOrDie((void *) Game_Header.Tags,
-                                      new_length*sizeof(*Game_Header.Tags));
-    for(i = Game_Header.header_tags_length; i < new_length; i++){
-        Game_Header.Tags[i] = NULL;
+    GameHeader.Tags = (char **) ReallocOrDie((void *) GameHeader.Tags,
+                                      new_length*sizeof(*GameHeader.Tags));
+    for(i = GameHeader.header_tags_length; i < new_length; i++){
+        GameHeader.Tags[i] = NULL;
     }
-    Game_Header.header_tags_length = new_length;
+    GameHeader.header_tags_length = new_length;
 }
 
         /* Try to open the given file. Error and exit on failure. */
 FILE *
-MustOpen(const char *filename,const char *mode)
+must_open_file(const char *filename,const char *mode)
 {   FILE *fp;
 
     fp = fopen(filename,mode);
@@ -135,22 +138,22 @@ MustOpen(const char *filename,const char *mode)
 void
 report_details(FILE *outfp)
 {
-      if(Game_Header.Tags[WHITE_TAG] != NULL){
-          fprintf(outfp,"%s - ",Game_Header.Tags[WHITE_TAG]);
+      if(GameHeader.Tags[WHITE_TAG] != NULL){
+          fprintf(outfp,"%s - ",GameHeader.Tags[WHITE_TAG]);
       }
-      if(Game_Header.Tags[BLACK_TAG] != NULL){
-          fprintf(outfp,"%s ",Game_Header.Tags[BLACK_TAG]);
+      if(GameHeader.Tags[BLACK_TAG] != NULL){
+          fprintf(outfp,"%s ",GameHeader.Tags[BLACK_TAG]);
       }
 
-      if(Game_Header.Tags[EVENT_TAG] != NULL){
-          fprintf(outfp,"%s ",Game_Header.Tags[EVENT_TAG]);
+      if(GameHeader.Tags[EVENT_TAG] != NULL){
+          fprintf(outfp,"%s ",GameHeader.Tags[EVENT_TAG]);
       }
-      if(Game_Header.Tags[SITE_TAG] != NULL){
-          fprintf(outfp,"%s ",Game_Header.Tags[SITE_TAG]);
+      if(GameHeader.Tags[SITE_TAG] != NULL){
+          fprintf(outfp,"%s ",GameHeader.Tags[SITE_TAG]);
       }
 
-      if(Game_Header.Tags[DATE_TAG] != NULL){
-          fprintf(outfp,"%s ",Game_Header.Tags[DATE_TAG]);
+      if(GameHeader.Tags[DATE_TAG] != NULL){
+          fprintf(outfp,"%s ",GameHeader.Tags[DATE_TAG]);
       }
       putc('\n',outfp);
       fflush(outfp);
@@ -168,7 +171,7 @@ check_result(char **Tags,const char *terminating_result)
         if((result_tag == NULL) || (*result_tag == '\0') ||
                     (strcmp(result_tag,"?") == 0)){
             /* Use a copy of terminating result. */
-            result_tag = StringCopy(terminating_result);
+            result_tag = copy_string(terminating_result);
             Tags[RESULT_TAG] = result_tag;
         }
         else if((result_tag != NULL) &&
@@ -195,7 +198,7 @@ static FILE *
 select_output_file(StateInfo *GameState,const char *eco)
 {   
     if(GameState->games_per_file > 0){
-        if((GameState->num_games_matched % GameState->games_per_file) == 0){
+        if(GameState->games_per_file == 1 || (GameState->num_games_matched % GameState->games_per_file) == 1){
               /* Time to open the next one. */
               char filename[50];
 
@@ -205,7 +208,7 @@ select_output_file(StateInfo *GameState,const char *eco)
               sprintf(filename,"%u%s",
                         GameState->next_file_number,
                         output_file_suffix(GameState->output_format));
-              GameState->outputfile = MustOpen(filename,"w");
+              GameState->outputfile = must_open_file(filename,"w");
               GameState->next_file_number++;
         }
     }
@@ -226,11 +229,21 @@ select_output_file(StateInfo *GameState,const char *eco)
     return GameState->outputfile;
 }
 
+  /*
+   * Conditions for finishing processing, other than all the input
+   * having been processed.
+   */
+static Boolean finished_processing(void)
+{
+    return (GlobalState.matching_game_number > 0 &&
+	  GlobalState.num_games_matched == GlobalState.matching_game_number);
+}
+
 static void
 ParseOptGameList(SourceFileType file_type)
 {   Move *move_list = NULL;
 
-    while(ParseGame(&move_list)){
+    while(ParseGame(&move_list) && !finished_processing()){
         if(file_type == NORMALFILE){
             DealWithGame(move_list);
         }
@@ -261,7 +274,7 @@ ParseOptGameList(SourceFileType file_type)
          */
 static Boolean
 ParseGame(Move **returned_move_list)
-{   Boolean something_found = FALSE;
+{   /* Boolean something_found = FALSE; */
     CommentList *prefix_comment;
     Move *move_list = NULL;
     char *result;
@@ -273,19 +286,19 @@ ParseGame(Move **returned_move_list)
     /* Assume that we won't return anything. */
     *returned_move_list = NULL;
     /* Skip over any junk between games. */
-    Symbol = skip_to_next_game(Symbol);
+    current_symbol = skip_to_next_game(current_symbol);
     prefix_comment = ParseOptCommentList();
     if(prefix_comment != NULL){
         /* Free this here, as it is hard to
          * know whether it belongs to the game or the file.
          * It is better to put game comments after the tags.
          */
-        something_found = TRUE;
+        /* something_found = TRUE; */
         free_comment_list(prefix_comment);
         prefix_comment = NULL;
     }
     if(ParseOptTagList()){
-        something_found = TRUE;
+        /* something_found = TRUE; */
     }
     /* @@@ Beware of comments and/or tags without moves. */
     move_list = ParseMoveList();
@@ -309,14 +322,14 @@ ParseGame(Move **returned_move_list)
         if(result != NULL){
             /* Append it to the last move. */
             last_move->terminating_result = result;
-            check_result(Game_Header.Tags,result);
+            check_result(GameHeader.Tags,result);
             *returned_move_list = move_list;
         }
         else{
             fprintf(GlobalState.logfile,"Missing result.\n");
             report_details(GlobalState.logfile);
         }
-        something_found = TRUE;
+        /* something_found = TRUE; */
     }
     else{
         /* @@@ Nothing to attach the comment to. */
@@ -329,13 +342,13 @@ ParseGame(Move **returned_move_list)
          * When outputting a game, the missing result in this case
          * will have to be supplied from the tags.
          */
-        check_result(Game_Header.Tags,result);
+        check_result(GameHeader.Tags,result);
         if(result != NULL){
             (void) free((void *)result);
         }
         *returned_move_list = NULL;
     }
-    return Symbol != EOF_TOKEN;
+    return current_symbol != EOF_TOKEN;
 }
 
 Boolean
@@ -348,20 +361,21 @@ ParseOptTagList(void)
     }
     if(something_found){
         /* Perform any consistency checks. */
-        if((Game_Header.Tags[SETUP_TAG] != NULL) &&
-                (strcmp(Game_Header.Tags[SETUP_TAG],"1") == 0)){
+        if((GameHeader.Tags[SETUP_TAG] != NULL) &&
+                (strcmp(GameHeader.Tags[SETUP_TAG],"1") == 0)){
             /* There must be a FEN_TAG to go with it. */
-            if(Game_Header.Tags[FEN_TAG] == NULL){
+            if(GameHeader.Tags[FEN_TAG] == NULL){
                  fprintf(GlobalState.logfile,
                         "Missing %s Tag to accompany %s Tag.\n",
                         tag_header_string(FEN_TAG),
                         tag_header_string(SETUP_TAG));
+                print_error_context(GlobalState.logfile);
             }
         }
     }
     prefix_comment = ParseOptCommentList();
     if(prefix_comment != NULL){
-        Game_Header.prefix_comment = prefix_comment;
+        GameHeader.prefix_comment = prefix_comment;
         something_found = TRUE;
     }
     return something_found;
@@ -371,15 +385,15 @@ Boolean
 ParseTag(void)
 {   Boolean TagFound = TRUE;
 
-    if(Symbol == TAG){
+    if(current_symbol == TAG){
         TagName tag_index = yylval.tag_index;
 
-        Symbol = next_token();
-        if(Symbol == STRING){
+        current_symbol = next_token();
+        if(current_symbol == STRING){
             char *tag_string = yylval.token_string;
 
-            if(tag_index < Game_Header.header_tags_length){
-                Game_Header.Tags[tag_index] = tag_string;
+            if(tag_index < GameHeader.header_tags_length){
+                GameHeader.Tags[tag_index] = tag_string;
             }
             else{
                 print_error_context(GlobalState.logfile);
@@ -388,18 +402,18 @@ ParseTag(void)
                         tag_index,tag_string);
                 exit(1);
             }
-            Symbol = next_token();
+            current_symbol = next_token();
         }
         else{
             print_error_context(GlobalState.logfile);
             fprintf(GlobalState.logfile,"Missing tag string.\n");
         }
     }
-    else if(Symbol == STRING){
+    else if(current_symbol == STRING){
         print_error_context(GlobalState.logfile);
         fprintf(GlobalState.logfile,"Missing tag for %s.\n",yylval.token_string);
         (void) free((void *)yylval.token_string);
-        Symbol = next_token();
+        current_symbol = next_token();
     }
     else{
         TagFound = FALSE;
@@ -467,7 +481,7 @@ static Move *
 ParseMoveUnit(void)
 {   Move *move_details = NULL;
 
-    if(Symbol == MOVE){
+    if(current_symbol == MOVE){
         move_details = yylval.move_details;
 
 	if(move_details->class == NULL_MOVE && RAV_level == 0) {
@@ -475,13 +489,13 @@ ParseMoveUnit(void)
 	    fprintf(GlobalState.logfile, "Null moves (--) only allowed in variations.\n");
 	}
 
-        Symbol = next_token();
-        if(Symbol == CHECK_SYMBOL){
+        current_symbol = next_token();
+        if(current_symbol == CHECK_SYMBOL){
             strcat((char *) move_details->move,"+");
-            Symbol = next_token();
+            current_symbol = next_token();
             /* Sometimes + is followed by #, so cover this case. */
-            if(Symbol == CHECK_SYMBOL){
-                Symbol = next_token();
+            if(current_symbol == CHECK_SYMBOL){
+                current_symbol = next_token();
             }
         }
         move_details->Comment = ParseOptCommentList();
@@ -493,7 +507,7 @@ static CommentList *
 ParseOptCommentList(void)
 {   CommentList *head = NULL, *tail = NULL;
 
-    while(Symbol == COMMENT){
+    while(current_symbol == COMMENT){
         if(head == NULL){
             head = tail = yylval.comment;
         }
@@ -501,7 +515,7 @@ ParseOptCommentList(void)
             tail->next = yylval.comment;
             tail = tail->next;
         }
-        Symbol = next_token();
+        current_symbol = next_token();
     }
     return head;
 }
@@ -510,8 +524,8 @@ Boolean
 ParseOptMoveNumber(void)
 {   Boolean something_found = FALSE;
 
-    if(Symbol == MOVE_NUMBER){
-        Symbol = next_token();
+    if(current_symbol == MOVE_NUMBER){
+        current_symbol = next_token();
         something_found = TRUE;
     }
     return something_found;
@@ -521,14 +535,14 @@ static StringList *
 ParseOptNAGList(void)
 {   StringList *nags = NULL;
  
-    while(Symbol == NAG){
+    while(current_symbol == NAG){
         if(GlobalState.keep_NAGs){
-            nags = SaveStringListItem(nags,yylval.token_string);
+            nags = save_string_list_item(nags,yylval.token_string);
         }
         else{
             (void) free((void *)yylval.token_string);
         }
-        Symbol = next_token();
+        current_symbol = next_token();
     }
     return nags;
 }
@@ -554,7 +568,7 @@ static Variation *
 ParseVariant(void)
 {   Variation *variation = NULL;
 
-    if(Symbol == RAV_START){
+    if(current_symbol == RAV_START){
         CommentList *prefix_comment;
         CommentList *suffix_comment;
         char *result = NULL;
@@ -563,7 +577,7 @@ ParseVariant(void)
         RAV_level++;
         variation = MallocOrDie(sizeof(Variation));
 
-        Symbol = next_token();
+        current_symbol = next_token();
         prefix_comment = ParseOptCommentList();
         if(prefix_comment != NULL){
         }
@@ -595,12 +609,13 @@ ParseVariant(void)
         else{
             /* Ok. */
         }
-        if(Symbol == RAV_END){
+        if(current_symbol == RAV_END){
             RAV_level--;
-            Symbol = next_token();
+            current_symbol = next_token();
         }
         else{
             fprintf(GlobalState.logfile,"Missing ')' to close variation.\n");
+            print_error_context(GlobalState.logfile);
         }
         suffix_comment = ParseOptCommentList();
         if(suffix_comment != NULL){
@@ -617,16 +632,16 @@ static char *
 ParseResult(void)
 {   char *result = NULL;
 
-    if(Symbol == TERMINATING_RESULT){
+    if(current_symbol == TERMINATING_RESULT){
         result = yylval.token_string;
         if(RAV_level == 0){
             /* In the interests of skipping any intervening material
              * between games, set the lookahead to a dummy token.
              */
-            Symbol = NO_TOKEN;
+            current_symbol = NO_TOKEN;
         }
         else{
-            Symbol = next_token();
+            current_symbol = next_token();
         }
     }
     return result;
@@ -639,17 +654,17 @@ setup_for_new_game(void)
     RAV_level = 0;
 }
 
-        /* Discard any data held in the Game_Header.Tags structure. */
+        /* Discard any data held in the GameHeader.Tags structure. */
 void
 free_tags(void)
-{ int tag;
+{   unsigned tag;
 
-  for(tag = 0; tag < Game_Header.header_tags_length; tag++){
-      if(Game_Header.Tags[tag] != NULL){
-          free(Game_Header.Tags[tag]);
-          Game_Header.Tags[tag] = NULL;
-      }
-  }
+    for(tag = 0; tag < GameHeader.header_tags_length; tag++){
+        if(GameHeader.Tags[tag] != NULL){
+            free(GameHeader.Tags[tag]);
+            GameHeader.Tags[tag] = NULL;
+        }
+    }
 }
 
         /* Discard data from a gathered game. */
@@ -731,7 +746,7 @@ free_move_list(Move *move_list)
          * return the head of the resulting list.
          */
 StringList *
-SaveStringListItem(StringList *list,const char *str)
+save_string_list_item(StringList *list,const char *str)
 {
     if(str != NULL){
       StringList *new_item;
@@ -779,7 +794,7 @@ append_comments_to_move(Move *move,CommentList *Comment)
     }
 }
 
-void
+static void
 DealWithGame(Move *move_list)
 {   Game current_game;
     /* Record whether the game has been printed or not.
@@ -794,12 +809,13 @@ DealWithGame(Move *move_list)
     GlobalState.num_games_processed++;
 
     /* Fill in the information currently known. */
-    current_game.tags = Game_Header.Tags;
-    current_game.tags_length = Game_Header.header_tags_length;
-    current_game.prefix_comment = Game_Header.prefix_comment;
+    current_game.tags = GameHeader.Tags;
+    current_game.tags_length = GameHeader.header_tags_length;
+    current_game.prefix_comment = GameHeader.prefix_comment;
     current_game.moves = move_list;
     current_game.moves_checked = FALSE;
     current_game.moves_ok = FALSE;
+    current_game.error_ply = 0;
   
     /* Determine whether or not this game is wanted, on the
      * basis of the various selection criteria available.
@@ -839,8 +855,23 @@ DealWithGame(Move *move_list)
             /* Don't output first occurrences. */
         }
         else if((original_filename == NULL) || !GlobalState.suppress_duplicates){
-            if(!GlobalState.check_only &&
-                        (GlobalState.current_file_type != CHECKFILE)){
+	    GlobalState.num_games_matched++;
+            if(GlobalState.check_only) {
+	        /* We are only checking. */
+		if(GlobalState.verbose){
+		    /* Report progress on logfile. */
+		    report_details(GlobalState.logfile);
+		}
+	    }
+	    else if(GlobalState.current_file_type == CHECKFILE){
+		/* We are only checking, so don't count this as a matched game. */
+		GlobalState.num_games_matched--;
+	    }
+	    else if(GlobalState.matching_game_number > 0 &&
+		  GlobalState.num_games_matched != GlobalState.matching_game_number) {
+	        /* This is not the right matching game to be output. */
+	    }
+	    else {
                 /* This game is to be kept and output. */
                 FILE *outputfile = select_output_file(&GlobalState,
                         current_game.tags[ECO_TAG]);
@@ -868,46 +899,36 @@ DealWithGame(Move *move_list)
                   print_str(outputfile," }");
                   terminate_line(outputfile);
                 }
-              /* Now output what we have. */
-              output_game(current_game,outputfile);
-              game_output = TRUE;
-            }
-          if(GlobalState.current_file_type == CHECKFILE){
-              /* Don't do anything else, but count this as a game output,
-               * so that it doesn't get caught by the non_matching_file
-               * option.
-               */
-              game_output = TRUE;
-          }
-          else{
-              /* Update the number of matched games. */
-              GlobalState.num_games_matched++;
-              if(GlobalState.verbose){
-                  /* Report progress on logfile. */
-                  report_details(GlobalState.logfile);
-              }
-          }
+		/* Now output what we have. */
+		output_game(current_game,outputfile);
+		game_output = TRUE;
+		if(GlobalState.verbose){
+		    /* Report progress on logfile. */
+		    report_details(GlobalState.logfile);
+		}
+	    }
         }
     }
-    if(!game_output && (GlobalState.non_matching_file != NULL)){
+    if(!game_output && (GlobalState.non_matching_file != NULL) &&
+    		GlobalState.current_file_type != CHECKFILE){
         /* The user wants to keep everything else. */
         if(!current_game.moves_checked){
              /* Make sure that the move text is in a reasonable state. */
              (void) apply_move_list(&current_game,&plycount);
         }
-        if(current_game.moves_ok){
+        if(current_game.moves_ok || GlobalState.keep_broken_games){
             output_game(current_game,GlobalState.non_matching_file);
         }
     }
   
     /* Game is finished with, so free everything. */
-    if(Game_Header.prefix_comment != NULL){
-        free_comment_list(Game_Header.prefix_comment);
+    if(GameHeader.prefix_comment != NULL){
+        free_comment_list(GameHeader.prefix_comment);
     }
-    /* Ensure that the Game_Header's prefix comment is NULL for
+    /* Ensure that the GameHeader's prefix comment is NULL for
      * the next game.
      */
-    Game_Header.prefix_comment = NULL;
+    GameHeader.prefix_comment = NULL;
   
     free_tags();
     free_move_list(current_game.moves);
@@ -925,12 +946,13 @@ DealWithEcoLine(Move *move_list)
     unsigned number_of_half_moves;
 
     /* Fill in the information currently known. */
-    current_game.tags = Game_Header.Tags;
-    current_game.tags_length = Game_Header.header_tags_length;
-    current_game.prefix_comment = Game_Header.prefix_comment;
+    current_game.tags = GameHeader.Tags;
+    current_game.tags_length = GameHeader.header_tags_length;
+    current_game.prefix_comment = GameHeader.prefix_comment;
     current_game.moves = move_list;
     current_game.moves_checked = FALSE;
     current_game.moves_ok = FALSE;
+    current_game.error_ply = 0;
   
     /* apply_eco_move_list checks out the moves.
      * It will also fill in the
@@ -946,13 +968,13 @@ DealWithEcoLine(Move *move_list)
     }
   
     /* Game is finished with, so free everything. */
-    if(Game_Header.prefix_comment != NULL){
-        free_comment_list(Game_Header.prefix_comment);
+    if(GameHeader.prefix_comment != NULL){
+        free_comment_list(GameHeader.prefix_comment);
     }
-    /* Ensure that the Game_Header's prefix comment is NULL for
+    /* Ensure that the GameHeader's prefix comment is NULL for
      * the next game.
      */
-    Game_Header.prefix_comment = NULL;
+    GameHeader.prefix_comment = NULL;
   
     free_tags();
     free_move_list(current_game.moves);
@@ -965,12 +987,16 @@ int
 yyparse(SourceFileType file_type)
 {
     setup_for_new_game();
-    Symbol = skip_to_next_game(NO_TOKEN);
+    current_symbol = skip_to_next_game(NO_TOKEN);
     ParseOptGameList(file_type);
-    if(Symbol == EOF_TOKEN){
+    if(current_symbol == EOF_TOKEN){
         /* Ok -- EOF. */
         return 0;
     }
+    else if(finished_processing()) {
+        /* Ok -- done all we need to. */
+	return 0;
+    }
     else{
         fprintf(GlobalState.logfile,"End of input reached before end of file.\n");
         return 1;
