@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -21,9 +21,8 @@
  */
 
 int yyparse(SourceFileType file_type);
-void DealWithGame(Move *move_list);
 void free_string_list(StringList *list);
 void init_game_header(void);
-void increase_game_header_tags_length(int new_length);
+void increase_game_header_tags_length(unsigned new_length);
 void report_details(FILE *outfp);
 void append_comments_to_move(Move *move,CommentList *Comment);
