@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -31,7 +31,7 @@ typedef struct HashLog {
     HashCode final_hash_value;
     HashCode cumulative_hash_value;
     /* Record the file list index for the file this game was first found in. */
-    int file_number;
+    unsigned file_number;
     struct HashLog *next;
 } HashLog;
 
