@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -56,7 +56,6 @@ static void save_move(const unsigned char *move);
 static void save_q_castle(void);
 static void save_k_castle(void);
 static void terminate_input(void);
-static char *next_input_line(FILE *fp);
 static Boolean extract_yytext(const unsigned char *symbol_start,
                               const unsigned char *linep);
 static Boolean open_input(const char *infile);
@@ -69,10 +68,6 @@ static unsigned RAV_level = 0;
 static unsigned char last_move[MAX_MOVE_LEN+1];
 /* How many games we have extracted from this file. */
 static unsigned games_in_file = 0;
-/* Turn off lexical error messages if we are skipping
- * over a game in error.
- */
-static Boolean skipping_game = FALSE;
 
 /* Provide an input file pointer.
  * This is intialised in init_lex_tables.
@@ -84,19 +79,6 @@ static FILE *yyin = NULL;
 static unsigned char yytext[MAX_YYTEXT+1];
 YYSTYPE yylval;
 
-        /* Define a type to make it easy to return where we are
-         * in the input line.  This type is returned from those functions
-         * that may modify the line/linep pair in next_token().
-         */
-typedef struct{
-    /* Start of the malloc'ed line. */
-    char *line;
-    /* The next char to access in line, if line is not NULL. */
-    unsigned char *linep;
-    /* What token resulted. */
-    TokenType token;
-} LinePair;
-
 #define MAX_CHAR 256
 #define ALPHA_DIST ('a'-'A')
 /* Table of symbol classifications. */
@@ -105,20 +87,20 @@ static TokenType ChTab[MAX_CHAR];
 static short MoveChars[MAX_CHAR];
 
 /* Define a table to hold the list of tag strings and the corresponding
- * TagName index. This is initialised in init_tag_list().
+ * TagName index. This is initialised in init_list_of_known_tags().
  */
 static const char **TagList;
-static int tag_list_length = 0;
+static unsigned tag_list_length = 0;
 
         /* Initialise the TagList. This should be stored in alphabetical order,
          * by virtue of the order in which the _TAG values are defined.
          */
 static void
-init_tag_list(void)
+init_list_of_known_tags(void)
 {
-    int i;
+    unsigned i;
     tag_list_length = ORIGINAL_NUMBER_OF_TAGS;
-    TagList = (const char * *) MallocOrDie(tag_list_length*sizeof(*TagList));
+    TagList = (const char **) MallocOrDie(tag_list_length*sizeof(*TagList));
     /* Be paranoid and put a string in every entry. */
     for(i = 0; i < tag_list_length; i++){
         TagList[i] = "";
@@ -139,6 +121,7 @@ init_tag_list(void)
     TagList[EVENT_SPONSOR_TAG] = "EventSponsor";
     TagList[FEN_TAG] = "FEN";
     TagList[PSEUDO_FEN_PATTERN_TAG] = "FENPattern";
+    TagList[HASHCODE_TAG] = "HashCode";
     TagList[LONG_ECO_TAG] = "LongECO";
     TagList[MODE_TAG] = "Mode";
     TagList[NIC_TAG] = "NIC";
@@ -155,6 +138,7 @@ init_tag_list(void)
     TagList[TERMINATION_TAG] = "Termination";
     TagList[TIME_TAG] = "Time";
     TagList[TIME_CONTROL_TAG] = "TimeControl";
+    TagList[TOTAL_PLY_COUNT_TAG] = "TotalPlyCount";
     TagList[UTC_DATE_TAG] = "UTCDate";
     TagList[UTC_TIME_TAG] = "UTCTime";
     TagList[VARIATION_TAG] = "Variation";
@@ -172,11 +156,11 @@ init_tag_list(void)
      */
 static TagName
 make_new_tag(const char *tag)
-{   int tag_index = tag_list_length;
+{   unsigned tag_index = tag_list_length;
     tag_list_length++;
     TagList = (const char **) ReallocOrDie((void *)TagList,
                     tag_list_length*sizeof(*TagList));
-    TagList[tag_index] = StringCopy(tag);
+    TagList[tag_index] = copy_string(tag);
     /* Ensure that the game header's tags array can accommodate
      * the new tag.
      */
@@ -187,7 +171,7 @@ make_new_tag(const char *tag)
 const char *
 tag_header_string(TagName tag)
 {
-    if((tag >= 0) && (tag < tag_list_length)){
+    if(tag < tag_list_length){
         return TagList[tag];
     }
     else{
@@ -208,7 +192,7 @@ init_lex_tables(void)
 
     /* Assume standard input will be used, until we know otherwise. */
     yyin = stdin;
-    init_tag_list();
+    init_list_of_known_tags();
     /* Initialise ChTab[]. */
     for(i = 0; i < MAX_CHAR; i++){
         ChTab[i] = ERROR_TOKEN;
@@ -312,7 +296,7 @@ init_lex_tables(void)
         /* Starting from linep in line, gather up the string until
          * the closing quote.  Skip over the closing quote.
          */
-static LinePair
+LinePair
 gather_string(char *line, unsigned char *linep)
 {   LinePair resulting_line;
     char ch;
@@ -345,9 +329,10 @@ gather_string(char *line, unsigned char *linep)
      */
     if(ch == '\0'){
         /* Too far. */
-        if(!skipping_game)
+        if(!GlobalState.skipping_current_game) {
             fprintf(GlobalState.logfile,"Missing closing quote in %s\n",line);
-        if(len != 1){
+	}
+        if(len > 1){
             /* Move back to the null. */
             linep--;
             str[len-1] = '\0';
@@ -362,6 +347,16 @@ gather_string(char *line, unsigned char *linep)
     return resulting_line;
 }
 
+	/*
+	 * Is ch of the given character class?
+	 * External access to ChTab.
+	 */
+Boolean
+is_character_class(unsigned char ch, TokenType character_class)
+{
+    return ChTab[ch] == character_class;
+}
+
         /* Starting from linep in line, gather up a comment until
          * the END_COMMENT.  Skip over the END_COMMENT.
          */
@@ -390,7 +385,7 @@ gather_comment(char *line, unsigned char *linep)
             comment_str = (char *)MallocOrDie(len+1);
             strncpy(comment_str,(const char *) (linep-len-1) ,len);
             comment_str[len] = '\0';
-            current_comment = SaveStringListItem(current_comment,comment_str);
+            current_comment = save_string_list_item(current_comment,comment_str);
         }
         if(ch == '\0'){
             line = next_input_line(yyin);
@@ -505,7 +500,7 @@ gather_possible_numeric(char *line, unsigned char *linep, char initial_digit)
      */
 static int
 identify_tag(const char *tag_string)
-{   int tag_index;
+{   unsigned tag_index;
 
     for(tag_index = 0; tag_index < tag_list_length; tag_index++){
         if(strcmp(tag_string,TagList[tag_index]) == 0){
@@ -519,14 +514,14 @@ identify_tag(const char *tag_string)
         /* Starting from linep in line, gather up the tag name.
          * Skip over any preceding white space.
          */
-static LinePair
+LinePair
 gather_tag(char *line, unsigned char *linep)
 {   LinePair resulting_line;
     char ch;
     unsigned len = 0;
 
     do{
-        /* Check for end of line whilst skipping white space. */
+        /* Check for end of line while skipping white space. */
         if(*linep == '\0'){
             line = next_input_line(yyin);
             linep = (unsigned char *) line;
@@ -559,7 +554,7 @@ gather_tag(char *line, unsigned char *linep)
             if(tag_item < 0){
                 tag_item = make_new_tag(tag_string);
             }
-            if(tag_item >= 0 && tag_item < tag_list_length){
+            if(tag_item >= 0 && ((unsigned) tag_item) < tag_list_length){
                 yylval.tag_index = tag_item;
                 resulting_line.token = TAG;
                 (void) free((void *)tag_string);
@@ -596,7 +591,7 @@ extract_yytext(const unsigned char *symbol_start,const unsigned char *linep)
     else{
         strncpy((char *) yytext,(const char *) symbol_start,MAX_YYTEXT);
         yytext[MAX_YYTEXT] = '\0';
-        if(!skipping_game)
+        if(!GlobalState.skipping_current_game)
             fprintf(GlobalState.logfile,"Symbol %s exceeds length of %u.\n",
                         yytext, MAX_YYTEXT);
         Ok = FALSE;
@@ -670,8 +665,8 @@ get_next_symbol(void)
                     token = resulting_line.token;
                     break;
                 case COMMENT_END:
-                    if(!skipping_game) {
-                        fprintf(GlobalState.logfile,"Unmatched comment end.\n");
+                    if(!GlobalState.skipping_current_game) {
+                        fprintf(GlobalState.logfile,"Unmatched comment end on line %lu.\n", line_number);
 		    }
                     token = NO_TOKEN;
                     break;
@@ -765,7 +760,7 @@ get_next_symbol(void)
                                 token = MOVE;
                             }
                             else{
-                                if(!skipping_game){
+                                if(!GlobalState.skipping_current_game){
                                     print_error_context(GlobalState.logfile);
                                     fprintf(GlobalState.logfile,
                                                 "Unknown move text %s.\n",yytext);
@@ -778,7 +773,7 @@ get_next_symbol(void)
                         }
                     }
                     else{
-                        if(!skipping_game){
+                        if(!GlobalState.skipping_current_game){
                             print_error_context(GlobalState.logfile);
                             fprintf(GlobalState.logfile,
                                     "Unknown character %c (Hex: %x).\n",
@@ -810,7 +805,7 @@ get_next_symbol(void)
                         RAV_level--;
                     }
                     else{
-                        if(!skipping_game){
+                        if(!GlobalState.skipping_current_game){
                             print_error_context(GlobalState.logfile);
                             fprintf(GlobalState.logfile,"Too many ')' found.\n");
                         }
@@ -840,7 +835,7 @@ get_next_symbol(void)
                     token = NO_TOKEN;
                     break;
                 case ERROR_TOKEN:
-                    if(!skipping_game){
+                    if(!GlobalState.skipping_current_game){
                         print_error_context(GlobalState.logfile);
                         fprintf(GlobalState.logfile,
                                 "Unknown character %c (Hex: %x).\n",
@@ -860,7 +855,7 @@ get_next_symbol(void)
                     token = NO_TOKEN;
                     break;
                 default:
-                    if(!skipping_game){
+                    if(!GlobalState.skipping_current_game){
                         print_error_context(GlobalState.logfile);
                         fprintf(GlobalState.logfile,
                             "Internal error: Missing case for %d on char %x.\n",
@@ -912,10 +907,10 @@ TokenType
 skip_to_next_game(TokenType token)
 {   
     if(skip_token(token)){
-        skipping_game = TRUE;
+        GlobalState.skipping_current_game = TRUE;
         do{
             if(token == COMMENT){
-                /* We ought to free the space. */
+                /* Free the space. */
                 if((yylval.comment != NULL) &&
                                 (yylval.comment->Comment != NULL)){
                     free_string_list(yylval.comment->Comment);
@@ -925,7 +920,7 @@ skip_to_next_game(TokenType token)
             }
             token = next_token();
         } while(skip_token(token));
-        skipping_game = FALSE;
+        GlobalState.skipping_current_game = FALSE;
     }
     return token;
 }
@@ -977,7 +972,7 @@ static FILE_LIST list_of_files = {
 };
 
     /* Return the index number of the current input file in list_of_files. */
-int
+unsigned
 current_file_number(void)
 {
     return current_file_num;
@@ -1084,7 +1079,7 @@ add_filename_to_source_list(const char *filename,SourceFileType file_type)
             }
         }
     }
-    list_of_files.files[location] = StringCopy(filename);
+    list_of_files.files[location] = copy_string(filename);
     list_of_files.file_type[location] = file_type;
     list_of_files.num_files++;
     /* Keep the list properly terminated. */
@@ -1160,17 +1155,14 @@ open_first_file(void)
          * file number.
          */
 const char *
-input_file_name(int file_number)
+input_file_name(unsigned file_number)
 {
     if(file_number >= list_of_files.num_files) {
         return NULL;
     }
-    else if(file_number >= 0){
+    else {
         return list_of_files.files[file_number];
     }
-    else{
-        return NULL;
-    }
 }
 
 
@@ -1185,7 +1177,7 @@ print_error_context(FILE *fp)
 }
 
 
-        /* Save the given str accessible to YACC. */
+        /* Make the given str accessible. */
 static void
 save_string(const char *str)
 {   const size_t len = strlen(str);
@@ -1197,7 +1189,7 @@ save_string(const char *str)
 }
 
         /* Return the next line of input from fp. */
-static char *
+char *
 next_input_line(FILE *fp)
 {   /* Retain each line in turn, so as to be able to free it. */
     static char *line = NULL;
@@ -1276,207 +1268,3 @@ terminate_input(void)
     }
 }
 
-        /* Read the list of extraction criteria from TagFile.
-         * This doesn't use the normal lexical analyser before the
-         * PGN files are processed but circumvents next_token by
-         * calling get_tag() and get_string. This allows it to detect
-         * EOF before yywrap() is called.
-         * Be careful to leave lex in the right state.
-         */
-void
-read_tag_file(const char *TagFile)
-{   
-    yyin = fopen(TagFile,"r");
-    if(yyin != NULL){
-        Boolean keep_reading = TRUE;
-        
-        while(keep_reading){
-            char *line = next_input_line(yyin);
-            if(line != NULL){
-                keep_reading = process_tag_line(TagFile,line);
-            }
-            else{
-                keep_reading = FALSE;
-            }
-        }
-        (void) fclose(yyin);
-        /* Call yywrap in order to set up for the next (first) input file. */
-        (void) yywrap();
-    }
-    else{
-        fprintf(GlobalState.logfile,
-                "Unable to open %s for reading.\n",TagFile);
-        exit(1);
-    }
-}
-
-        /* Read the contents of a file that lists the
-         * required output ordering for tags.
-         */
-void
-read_tag_roster_file(const char *RosterFile)
-{   Boolean keep_reading = TRUE;
-    yyin = MustOpen(RosterFile,"r");
-    
-    while(keep_reading){
-        char *line = next_input_line(yyin);
-        if(line != NULL){
-            keep_reading = process_roster_line(line);
-        }
-        else{
-            keep_reading = FALSE;
-        }
-    }
-    (void) fclose(yyin);
-    /* Call yywrap in order to set up for the next (first) input file. */
-    (void) yywrap();
-}
-
-        /* Extract a tag/value pair from the given line.
-         * Return TRUE if this was successful.
-         */
-Boolean
-process_tag_line(const char *TagFile,char *line)
-{
-    Boolean keep_reading = TRUE;
-    if(non_blank_line(line)){
-        unsigned char *linep = (unsigned char *) line;
-        /* We should find a tag. */
-        LinePair resulting_line = gather_tag(line,linep);
-        TokenType tag_token;
-
-        /* Pick up where we are now. */
-        line = resulting_line.line;
-        linep = resulting_line.linep;
-        tag_token = resulting_line.token;
-        if(tag_token != NO_TOKEN){
-            /* Pick up which tag it was. */
-            int tag_index = yylval.tag_index;
-            /* Allow for an optional operator. */
-            TagOperator operator = NONE;
-
-            /* Skip whitespace up to a double quote. */
-            while(ChTab[(unsigned)*linep] == WHITESPACE){
-                linep++;
-            }
-            /* Allow for an optional operator. */
-            if(ChTab[(unsigned)*linep] == OPERATOR){
-                switch(*linep){
-                  case '<':
-                    linep++;
-                    if(*linep == '='){
-                        linep++;
-                        operator = LESS_THAN_OR_EQUAL_TO;
-                    }
-                    else if(*linep == '>'){
-                        linep++;
-                        operator = NOT_EQUAL_TO;
-                    }
-                    else{
-                        operator = LESS_THAN;;
-                    }
-                    break;
-                  case '>':
-                    linep++;
-                    if(*linep == '='){
-                        linep++;
-                        operator = GREATER_THAN_OR_EQUAL_TO;
-                    }
-                    else{
-                        operator = GREATER_THAN;
-                    }
-                    break;
-                  case '=':
-                    linep++;
-                    operator = EQUAL_TO;
-                    break;
-                  default:
-                    fprintf(GlobalState.logfile,
-                        "Internal error: unknown operator in %s\n",line);
-                    linep++;
-                    break;
-                }
-                /* Skip whitespace up to a double quote. */
-                while(ChTab[(unsigned)*linep] == WHITESPACE){
-                    linep++;
-                }
-            }
-
-            if(ChTab[(unsigned)*linep] == DOUBLE_QUOTE){
-                /* A string, as expected. */
-                linep++;
-                resulting_line = gather_string(line,linep);
-                line = resulting_line.line;
-                linep = resulting_line.linep;
-                if(tag_token == TAG){
-                    /* Treat FEN* tags as a special case.
-                     * Use the position they represent to indicate
-                     * a positional match.
-                     */
-                    if(tag_index == FEN_TAG){
-                        add_fen_positional_match(yylval.token_string);
-                        (void) free((void *)yylval.token_string);
-                    }
-                    else if(tag_index == PSEUDO_FEN_PATTERN_TAG){
-                        add_fen_pattern_match(yylval.token_string);
-                        (void) free((void *)yylval.token_string);
-                    }
-                    else{
-                        add_tag_to_list(tag_index,yylval.token_string,operator);
-                        (void) free((void *)yylval.token_string);
-                    }
-                }
-                else{
-                    if(!skipping_game){
-                            fprintf(GlobalState.logfile,
-                                "File %s: unrecognised tag name %s\n",
-                                        TagFile,line);
-                    }
-                    (void) free((void *)yylval.token_string);
-                }
-            }
-            else{
-                if(!skipping_game){
-                    fprintf(GlobalState.logfile,
-                            "File %s: missing quoted tag string in %s at %s\n",
-                            TagFile,line,linep);
-                }
-            }
-        }
-        else{
-            /* Terminate the reading, as we have run out of tags. */
-            keep_reading = FALSE;
-        }
-    }
-    return keep_reading;
-}
-
-        /* Extract a tag name from the given line.
-         * Return TRUE if this was successful.
-         */
-Boolean
-process_roster_line(char *line)
-{
-    Boolean keep_reading = TRUE;
-    if(non_blank_line(line)){
-        unsigned char *linep = (unsigned char *) line;
-        /* We should find a tag. */
-        LinePair resulting_line = gather_tag(line,linep);
-        TokenType tag_token;
-
-        /* Pick up where we are now. */
-        line = resulting_line.line;
-        linep = resulting_line.linep;
-        tag_token = resulting_line.token;
-        if(tag_token != NO_TOKEN){
-            /* Pick up which tag it was. */
-            int tag_index = yylval.tag_index;
-            add_to_output_tag_order((TagName) tag_index);
-        }
-        else{
-            /* Terminate the reading, as we have run out of tags. */
-            keep_reading = FALSE;
-        }
-    }
-    return keep_reading;
-}
