@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -20,6 +20,19 @@
  *
  */
 
+        /* Define a type to make it easy to return where we are
+         * in the input line.  This type is returned from those functions
+         * that may modify the line/linep pair in next_token().
+         */
+typedef struct{
+    /* Start of the malloc'ed line. */
+    char *line;
+    /* The next char to access in line, if line is not NULL. */
+    unsigned char *linep;
+    /* What token resulted. */
+    TokenType token;
+} LinePair;
+
 /* Define a type to hold a list of input files. */
 typedef struct {
     /* The list should have a (char *)NULL on the end. */
@@ -57,20 +70,20 @@ void yyerror(const char *s);
 void save_assessment(const char *assess);
 void restart_lex_for_new_game(void);
 void free_move_list(Move *move_list);
-void read_tag_file(const char *TagFile);
-void read_tag_roster_file(const char *RosterFile);
-Boolean process_tag_line(const char *TagFile,char *line);
-Boolean process_roster_line(char *line);
 void print_error_context(FILE *fp);
 void init_lex_tables(void);
 TokenType next_token(void);
 TokenType skip_to_next_game(TokenType token);
 const char *tag_header_string(TagName tag);
 Boolean open_first_file(void);
-const char *input_file_name(int file_number);
-int current_file_number(void);
+const char *input_file_name(unsigned file_number);
+unsigned current_file_number(void);
 Boolean open_eco_file(const char *eco_file);
 int yywrap(void);
 void add_filename_to_source_list(const char *filename,SourceFileType file_type);
 void add_filename_list_from_file(FILE *fp,SourceFileType file_type);
 void reset_line_number(void);
+char *next_input_line(FILE *fp);
+LinePair gather_tag(char *line, unsigned char *linep);
+LinePair gather_string(char *line, unsigned char *linep);
+Boolean is_character_class(unsigned char ch, TokenType character_class);
