@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -335,6 +335,9 @@ extract_tag_argument(const char *argstr)
         case 'e':
             add_tag_to_list(ECO_TAG,arg,NONE);
             break;
+        case 'h':
+            add_tag_to_list(HASHCODE_TAG,arg,NONE);
+            break;
         case 'p':
             add_tag_to_list(PSEUDO_PLAYER_TAG,arg,NONE);
             break;
@@ -369,10 +372,13 @@ extract_tag_argument(const char *argstr)
 static Boolean
 CheckDate(const char *date_string,StringArray *list)
 {   unsigned list_index;
-    Boolean wanted = TRUE;
 
-    for(list_index = 0; (list_index < list->num_used_elements) && wanted;
-                                list_index++){
+    /* Care needed because dates with operators must be ANDed and
+     * dates without operators must be ORed.
+     * The first match is used to set wanted properly for the first time.
+     */
+    Boolean wanted = FALSE;
+    for(list_index = 0; list_index < list->num_used_elements; list_index++){
         const char *list_string = list->tag_strings[list_index].tag_string;
         TagOperator operator = list->tag_strings[list_index].operator;
 
@@ -394,37 +400,50 @@ CheckDate(const char *date_string,StringArray *list)
             if((sscanf(list_string,"%u",&list_date) == 1) &&
                   (sscanf(date_string,"%u",&game_date) == 1) &&
                   (game_date > MINDATE) && (game_date < MAXDATE)){
+		Boolean matches;
                 switch(operator){
                     case LESS_THAN:
-                        wanted = game_date < list_date;
+                        matches = game_date < list_date;
                         break;
                     case LESS_THAN_OR_EQUAL_TO:
-                        wanted = game_date <= list_date;
+                        matches = game_date <= list_date;
                         break;
                     case GREATER_THAN:
-                        wanted = game_date > list_date;
+                        matches = game_date > list_date;
                         break;
                     case GREATER_THAN_OR_EQUAL_TO:
-                        wanted = game_date >= list_date;
+                        matches = game_date >= list_date;
                         break;
                     case EQUAL_TO:
-                        wanted = game_date == list_date;
+                        matches = game_date == list_date;
                         break;
                     case NOT_EQUAL_TO:
-                        wanted = game_date != list_date;
+                        matches = game_date != list_date;
                         break;
                     case NONE:
-                        /* Already covered above. */
+		    default:
+                        /* Should have been covered above. */
+			matches = FALSE;
                         break;
                 }
+		if(list_index == 0) {
+		    wanted = matches;
+		}
+		else {
+		    wanted = wanted && matches;
+		}
             }
             else{
                 /* Bad format, or out of range. Assume not wanted. */
+		wanted = FALSE;
             }
         }
         else{
-            /* Just a straight prefix match. */
-            wanted = strncmp(date_string,list_string,strlen(list_string)) == 0;
+	    /* No need to check if we already have a match. */
+	    if(list_index == 0 || !wanted) {
+		/* Just a straight prefix match. */
+	        wanted = strncmp(date_string,list_string,strlen(list_string)) == 0;
+	    }
         }
     }
     return wanted;
