@@ -59,6 +59,7 @@
      * C++ there would have been a cleaner interface!
      */
 StateInfo GlobalState = {
+    FALSE,                /* skipping_current_game */
     FALSE,                /* check_only (-r) */
     TRUE,                 /* verbose (-s) */
     TRUE,                 /* keep_NAGs (-N) */
@@ -86,6 +87,8 @@ StateInfo GlobalState = {
     TRUE,                 /* keep_results (--noresults) */
     TRUE,                 /* keep_checks (--nochecks) */
     FALSE,                /* output_evaluation (--evaluation) */
+    FALSE,                /* keep_broken_games (--keepbroken) */
+    FALSE,                /* suppress_redundant_ep_info (--nofauxep) */
     0,                    /* depth_of_positional_search */
     0,                    /* num_games_processed */
     0,                    /* num_games_matched */
@@ -93,10 +96,13 @@ StateInfo GlobalState = {
     1,                    /* next_file_number */
     0,                    /* lower_move_bound */
     10000,                /* upper_move_bound */
-    -1,                    /* output_ply_limit (--plylimit) */
+    -1,                   /* output_ply_limit (--plylimit) */
+    0,                    /* matching_game_number */
     FALSE,                /* output_FEN_string */
     FALSE,                /* add_FEN_comments (--fencomments) */
     FALSE,                /* add_position_match_comments (--markmatches) */
+    FALSE,                /* output_total_plycount (--totalplycount) */
+    FALSE,                /* add_hashcode_tag (--addhashcode) */
     "MATCH",		  /* position_match_comment (--markpositionmatches) */
     (char *)NULL,         /* current_input_file */
     NORMALFILE,           /* current_file_type */
@@ -123,7 +129,7 @@ main(int argc, char *argv[])
 
     /* Prepare global state. */
     init_default_global_state();
-    /* Prepare the GameHeader. */
+    /* Prepare the Game_Header. */
     init_game_header();
     /* Prepare the tag lists for -t/-T matching. */
     init_tag_lists();
