@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -217,7 +217,7 @@ piece_is_colour(Piece coloured_piece, Colour colour)
          * the full set of changes implied by it.
          */
 void
-make_move(Col from_col, Rank from_rank, Col to_col, Rank to_rank,
+make_move(MoveClass class, Col from_col, Rank from_rank, Col to_col, Rank to_rank,
           Piece piece, Colour colour,Board *board)
 {   short to_r = RankConvert(to_rank);
     short to_c = ColConvert(to_col);
@@ -318,10 +318,20 @@ make_move(Col from_col, Rank from_rank, Col to_col, Rank to_rank,
         }
     }
     /* Clear the source square. */
+    /* NB: Up to v17-18 there was an error here in the calculation of the hash
+     * when the move class is PAWN_MOVE_WITH_PROMOTION.
+     * The value of the promoted piece was removed, rather than the value
+     * of the pawn because piece is the promoted piece rather than PAWN.
+     */
+    if(class == PAWN_MOVE_WITH_PROMOTION && piece != PAWN) {
+	/* Remove the promoted pawn. */
+	board->hash_value ^= HashLookup(from_col,from_rank,PAWN,colour);
+    }
+    else {
+	board->hash_value ^= HashLookup(from_col,from_rank,piece,colour);
+    }
     board->board[from_r][from_c] = EMPTY;
-    board->hash_value ^= HashLookup(from_col,from_rank,piece,colour);
     if(board->board[to_r][to_c] != EMPTY){
-
         /* Delete the captured piece from the hash value. */
         Piece coloured_piece = board->board[to_r][to_c];
         Piece removed_piece;
@@ -364,12 +374,24 @@ make_move(Col from_col, Rank from_rank, Col to_col, Rank to_rank,
         capture = TRUE;
     }
     /* Deal with the half-move clock. */
-    if(capture || piece == PAWN) {
+    if(piece == PAWN || class == PAWN_MOVE_WITH_PROMOTION) {
         board->halfmove_clock = 0;
     }
-    else {
+    else if(capture) {
+        board->halfmove_clock = 0;
+    }
+    else if(piece != ROOK) {
         board->halfmove_clock++;
     }
+    else {
+        /* Check whether this is the Rook being moved as part of a
+         * two-stage castling move. If so, then don't update the
+         * clock a second time.
+         */
+        if(class != KINGSIDE_CASTLE && class != QUEENSIDE_CASTLE) {
+            board->halfmove_clock++;
+        }
+    }
     /* Place the piece at its destination. */
     board->board[to_r][to_c] = MAKE_COLOURED_PIECE(colour,piece);
     /* Insert the moved piece into the hash value. */
@@ -740,7 +762,7 @@ exclude_checks(Piece piece, Colour colour,MovePair *possibles, const Board *boar
     for(move = possibles; move != NULL; ){
         /* Take a copy of the board before playing this next move. */
         copy_board = *board;
-        make_move(move->from_col,move->from_rank,
+        make_move(UNKNOWN_MOVE, move->from_col,move->from_rank,
                   move->to_col,move->to_rank,piece,colour,&copy_board);
         if(king_is_in_check(&copy_board,colour) != NOCHECK){
             MovePair *illegal_move = move;
@@ -1763,7 +1785,7 @@ king_is_in_checkmate(Colour colour,Board *board)
                        break;
                    default:
                        fprintf(GlobalState.logfile,
-                           "Internal error: unknown piece %d in king_is_in_checmate().\n",
+                           "Internal error: unknown piece %d in king_is_in_checkmate().\n",
                                piece);
                 }
            }
@@ -1824,7 +1846,7 @@ approx_how_many_moves(Board *board)
                        break;
                    default:
                        fprintf(GlobalState.logfile,
-                           "Internal error: unknown piece %d in king_is_in_checmate().\n",
+                           "Internal error: unknown piece %d in king_is_in_checkmate().\n",
                                piece);
                 }
                if(moves != NULL){
@@ -1906,7 +1928,7 @@ find_all_moves(const Board *board, Colour colour)
                        break;
                    default:
                        fprintf(GlobalState.logfile,
-                           "Internal error: unknown piece %d in king_is_in_checmate().\n",
+                           "Internal error: unknown piece %d in king_is_in_checkmate().\n",
                                piece);
                 }
                if(moves != NULL){
