@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -23,7 +23,7 @@
 void init_hashtab(void);
 Boolean determine_move_details(Colour colour,Move *move_details, Board *board);
 HashCode HashLookup(Col col, Rank rank, Piece piece, Colour colour);
-void make_move(Col from_col, Rank from_rank, Col to_col, Rank to_rank,
+void make_move(MoveClass class, Col from_col, Rank from_rank, Col to_col, Rank to_rank,
                 Piece piece, Colour colour,Board *board);
 CheckStatus king_is_in_check(const Board *board,Colour king_colour);
 MovePair *find_pawn_moves(Col from_col, Rank from_rank, Col to_col,Rank to_rank,
