@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -69,6 +69,10 @@ static void print_epd_move_list(Game current_game,FILE *outputfile,
                                 unsigned move_number, Boolean white_to_move,
                                 Board *final_board);
 static const char *build_FEN_comment(const Board *board);
+static void add_total_plycount(const Game *game, Boolean count_variations);
+static void add_hashcode_tag(const Game *game);
+static unsigned count_single_move_ply(const Move *move_details, Boolean count_variations);
+static unsigned count_move_list_ply(Move *move_list, Boolean count_variations);
 
         /* List, the order in which the tags should be output.
          * The first seven should be the Seven Tag Roster that should
@@ -143,6 +147,8 @@ which_output_format(const char *arg)
         { "HALG", HALG },
         { "ELALG", ELALG },
         { "elalg", ELALG },
+        { "XLALG", XLALG },
+        { "xlalg", XLALG },
         { "uci", UCI },
         { "cm", CM },
         { "", SOURCE },
@@ -165,7 +171,7 @@ which_output_format(const char *arg)
              * a 6-piece suffix listing language-specific
              * letters to use in the output.
              */
-            if((format == SAN || format == ELALG) &&
+            if((format == SAN || format == ELALG || format == XLALG) &&
                    (strlen(arg) > format_prefix_len)){
                 set_output_piece_characters(&arg[format_prefix_len]);
             }
@@ -191,6 +197,7 @@ output_file_suffix(OutputFormat format)
         case LALG:
         case HALG:
         case ELALG:
+        case XLALG:
             return PGN_suffix;
         case EPD:
             return EPD_suffix;
@@ -411,7 +418,7 @@ print_comment_list(FILE *fp, CommentList *comment_list)
             single_char(fp,'{');
             for( ; comment != NULL; comment = comment->next){
 		/* Make a copy because the string is altered. */
-		char *str = StringCopy(comment->str);
+		char *str = copy_string(comment->str);
                 chunk = strtok(str," ");
                 while(chunk != NULL){
                     print_separator(fp);
@@ -435,7 +442,7 @@ print_move_list(FILE *outputfile, unsigned move_number, Boolean white_to_move,
 {   unsigned print_move_number = 1;
     const Move *move = move_details;
     Boolean keepPrinting;
-    unsigned plies;
+    int plies;
 
     /* Work out the ply depth. */
     plies = 2*(move_number) - 1;
@@ -499,7 +506,9 @@ print_move_list(FILE *outputfile, unsigned move_number, Boolean white_to_move,
     }
 
     if(move != NULL && !keepPrinting) {
-        /* We ended printing the game prematurely.
+        /* We ended printing the game prematurely. */
+
+	/*
          * Decide whether to print a result indicator.
          */
         if(GlobalState.keep_results) {
@@ -525,6 +534,7 @@ static Boolean
 print_move(FILE *outputfile, unsigned move_number, unsigned print_move_number,
                 Boolean white_to_move, const Move *move_details)
 {   Boolean something_printed = FALSE;
+    OutputFormat output_format = GlobalState.output_format;
 
     if(move_details == NULL){
         /* Shouldn't happen. */
@@ -551,7 +561,7 @@ print_move(FILE *outputfile, unsigned move_number, unsigned print_move_number,
                     print_str(outputfile,small_number);
                     print_separator(outputfile);
                 }
-                switch(GlobalState.output_format){
+                switch(output_format){
                     case SAN:
                     case SOURCE:
                         /* @@@ move_text should be handled as unsigned
@@ -598,7 +608,7 @@ print_move(FILE *outputfile, unsigned move_number, unsigned print_move_number,
                                        move_details->to_rank);
                                    break;
                                case PAWN_MOVE_WITH_PROMOTION:
-                                   sprintf(algebraic,"%c%c-%c%c%c",
+                                   sprintf(algebraic,"%c%c%-c%c%c",
                                        move_details->from_col,
                                        move_details->from_rank,
                                        move_details->to_col,
@@ -629,11 +639,12 @@ print_move(FILE *outputfile, unsigned move_number, unsigned print_move_number,
                         break;
                     case LALG:
                     case ELALG:
+                    case XLALG:
                         {  char algebraic[MAX_MOVE_LEN+1];
                            size_t ind = 0;
 
                            /* Prefix with a piece name if ELALG. */
-                           if(GlobalState.output_format == ELALG &&
+                           if((output_format == ELALG || output_format == XLALG) &&
                                         move_details->class == PIECE_MOVE){
                                strcpy(algebraic,
                                       piece_str(move_details->piece_to_move));
@@ -641,12 +652,26 @@ print_move(FILE *outputfile, unsigned move_number, unsigned print_move_number,
                            }
                            /* Format the basics. */
 			   if(move_details->class != NULL_MOVE) {
-			       sprintf(&algebraic[ind],"%c%c%c%c",
+			       sprintf(&algebraic[ind],"%c%c",
 					   move_details->from_col,
-					   move_details->from_rank,
+					   move_details->from_rank);
+			       ind += 2;
+			       if(output_format == XLALG) {
+			          /* Add a separating - or x. */
+				  char separator;
+				  if(move_details->captured_piece != EMPTY) {
+				      separator = 'x';
+				  }
+				  else {
+				      separator = '-';
+				  }
+				 sprintf(&algebraic[ind],"%c",separator);
+				 ind++;
+			       }
+			       sprintf(&algebraic[ind],"%c%c",
 					   move_details->to_col,
 					   move_details->to_rank);
-			       ind += 4;
+			       ind += 2;
 			   }
 			   else {
 			       strcpy(algebraic, NULL_MOVE_STRING);
@@ -661,7 +686,7 @@ print_move(FILE *outputfile, unsigned move_number, unsigned print_move_number,
                                    /* Nothing more to do at this stage. */
                                    break;
                                case ENPASSANT_PAWN_MOVE:
-                                   if(GlobalState.output_format == ELALG){
+                                   if(output_format == ELALG){
                                        strcat(algebraic,"ep");
                                        ind += 2;
                                    }
@@ -696,7 +721,7 @@ print_move(FILE *outputfile, unsigned move_number, unsigned print_move_number,
                     default:
                         fprintf(GlobalState.logfile,
                                 "Unknown output format %d in print_move()\n",
-                                GlobalState.output_format);
+                                output_format);
                                 exit(1);
                         break;
                 }
@@ -793,7 +818,7 @@ output_cm_comment(CommentList *comment,FILE *outputfile,unsigned indent)
         StringList *comment_str = comment->Comment;
         
         for( ; comment_str != NULL; comment_str = comment_str->next){
-	    char *str = StringCopy(comment_str->str);
+	    char *str = copy_string(comment_str->str);
             chunk = strtok(str," ");
             while(chunk != NULL){
                 size_t len = strlen(chunk);
@@ -934,7 +959,6 @@ output_cm_game(FILE *outputfile,unsigned move_number,
     putc('\n',outputfile);
 }
 
-
         /* Output the current game according to the required output format. */
 void
 output_game(Game current_game,FILE *outputfile)
@@ -966,33 +990,39 @@ output_game(Game current_game,FILE *outputfile)
         /* Rewrite the moves of the game into
          * SAN (Standard Algebraic Notation).
          */
-        final_board = rewrite_game(current_game.moves,
-                                   current_game.tags[FEN_TAG]);
+        final_board = rewrite_game(&current_game);
     }
-    switch(GlobalState.output_format){
-        case SAN:
-        case SOURCE:
-        case LALG:
-        case HALG:
-        case ELALG:
-            print_algebraic_game(current_game,outputfile,move_number,white_to_move,
-                                 final_board);
-            break;
-        case EPD:
-            print_epd_game(current_game,outputfile,move_number,white_to_move,
-                           final_board);
-            break;
-        case CM:
-            output_cm_game(outputfile,move_number,white_to_move,current_game);
-            break;
-        default:
-            fprintf(GlobalState.logfile,
-                        "Internal error: unknown output type %d in output_game().\n",
-                        GlobalState.output_format);
-            break;
-    }
-    fflush(outputfile);
-    if(final_board != NULL){
+    if(final_board != NULL) {
+	if(GlobalState.output_total_plycount) {
+	    add_total_plycount(&current_game, GlobalState.keep_variations);
+	}
+	if(GlobalState.add_hashcode_tag || current_game.tags[HASHCODE_TAG] != NULL) {
+	    add_hashcode_tag(&current_game);
+	}
+	switch(GlobalState.output_format){
+	    case SAN:
+	    case SOURCE:
+	    case LALG:
+	    case HALG:
+	    case ELALG:
+	    case XLALG:
+		print_algebraic_game(current_game,outputfile,move_number,white_to_move,
+				     final_board);
+		break;
+	    case EPD:
+		print_epd_game(current_game,outputfile,move_number,white_to_move,
+			       final_board);
+		break;
+	    case CM:
+		output_cm_game(outputfile,move_number,white_to_move,current_game);
+		break;
+	    default:
+		fprintf(GlobalState.logfile,
+			    "Internal error: unknown output type %d in output_game().\n",
+			    GlobalState.output_format);
+		break;
+	}
+	fflush(outputfile);
         (void) free((void *) final_board);
     }
 }
@@ -1012,7 +1042,7 @@ add_to_output_tag_order(TagName tag)
      * for this tag.
      */
     for(tag_index = 0; (TagOrder[tag_index] != -1) && 
-                       (TagOrder[tag_index] != tag); tag_index++){
+                       (TagOrder[tag_index] != (int) tag); tag_index++){
     }
 
     if(TagOrder[tag_index] == -1){
@@ -1205,3 +1235,65 @@ build_FEN_comment(const Board *board)
     return fen;
 }
 
+/*
+ * Count how many ply recorded for the given move.
+ * Include variations if count_variations.
+ */
+static unsigned count_single_move_ply(const Move *move_details, Boolean count_variations)
+{
+    unsigned count = 1;
+    if(count_variations) {
+	Variation *variant = move_details->Variants;
+	while(variant != NULL) {
+	    count += count_move_list_ply(variant->moves, count_variations);
+	    variant = variant->next;
+	}
+    }
+    return count;
+}
+
+/*
+ * Count how many plies in the game in total.
+ * Include variations if count_variations.
+ */
+static unsigned count_move_list_ply(Move *move_list, Boolean count_variations)
+{
+    unsigned count = 0;
+    while(move_list != NULL) {
+        count += count_single_move_ply(move_list, count_variations);
+	move_list = move_list->next;
+    }
+    return count;
+}
+
+/*
+ * Count how many plies in the game in total.
+ * Include variations if count_variations.
+ */
+static void add_total_plycount(const Game *game, Boolean count_variations)
+{
+    unsigned count = count_move_list_ply(game->moves, count_variations);
+    char formatted_count[20];
+    sprintf(formatted_count, "%u", count);
+
+    if(game->tags[TOTAL_PLY_COUNT_TAG] != NULL) {
+        (void) free(game->tags[TOTAL_PLY_COUNT_TAG]);
+    }
+    game->tags[TOTAL_PLY_COUNT_TAG] = copy_string(formatted_count);
+}
+
+/*
+ * Include a tag containing a hashcode for the game.
+ * Use the cumulative hash value.
+ */
+static void add_hashcode_tag(const Game *game)
+{
+    HashCode hashcode = game->cumulative_hash_value;
+    char formatted_code[20];
+    sprintf(formatted_code, "%08x", (unsigned) hashcode);
+
+    if(game->tags[HASHCODE_TAG] != NULL) {
+        (void) free(game->tags[HASHCODE_TAG]);
+    }
+    game->tags[HASHCODE_TAG] = copy_string(formatted_code);
+}
