@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -52,6 +52,7 @@ typedef enum {
      * argument so that it becomes possible to indicate a FEN-based board pattern.
      */
     PSEUDO_FEN_PATTERN_TAG,
+    HASHCODE_TAG,
     LONG_ECO_TAG,
     MODE_TAG,
     NIC_TAG,
@@ -61,6 +62,10 @@ typedef enum {
      */
     PSEUDO_PLAYER_TAG,
     PLY_COUNT_TAG,
+    /* The TOTAL_PLY_COUNT_TAG is used with the --totalplycount argument
+     * so record the total number of plies in a game.
+     */
+    TOTAL_PLY_COUNT_TAG,
     RESULT_TAG,
     ROUND_TAG,
     SECTION_TAG,
