@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2013 David Barnes
+ *  Copyright (C) 1994-2015 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -31,9 +31,12 @@
      *     HALG: Hyphenated long-algebraic, e.g. e2-e4.
      *     ELALG: Enhanced long-algebraic. Includes piece names, e.g. Ng1f3,
      *            and en-passant notation.
+     *     XLALG: Enhanced long-algebraic. Includes piece names, e.g. Ng1f3,
+     *            en-passant notation and either - or x between squares for
+     *            non-capture and capture moves respectively.
      *     UCI: UCI-compatible format - actually LALG.
      */
-typedef enum { SOURCE, SAN, EPD, CM, LALG, HALG, ELALG, UCI } OutputFormat;
+typedef enum { SOURCE, SAN, EPD, CM, LALG, HALG, ELALG, XLALG, UCI } OutputFormat;
 
     /* Define a type to specify whether a move gives check, checkmate,
      * or nocheck.
@@ -51,7 +54,7 @@ typedef struct string_list {
 } StringList;
 
 /* The following function is used for linking list items together. */
-StringList *SaveStringListItem(StringList *list,const char *str);
+StringList *save_string_list_item(StringList *list,const char *str);
 
 typedef struct comment_list{
     StringList *Comment;
@@ -133,6 +136,10 @@ typedef struct {
     Boolean moves_checked;
     /* Whether the moves are ok, or not. */
     Boolean moves_ok;
+    /* if !moves_ok, the first ply at which an error was found.
+     * 0 => no error found.
+     */
+    int error_ply;
 } Game;
 
 /* Define a type to distinguish between CHECK files, NORMAL files,
@@ -167,6 +174,10 @@ typedef enum {
  * Most of these fields are set from the program's arguments.
  */
 typedef struct {
+    /* Whether we are skipping the current game - typically because
+     * of an error in its text.
+     */
+    Boolean skipping_current_game;
     /* Whether to check, but not write the converted output. */
     Boolean check_only;
     /* Whether to print a running commentary to logfile. */
@@ -221,6 +232,10 @@ typedef struct {
     Boolean keep_checks;
     /* Whether to output an evaluation value after each move. */
     Boolean output_evaluation;
+    /* Whether to keep games which have incorrect moves. */
+    Boolean keep_broken_games;
+    /* Whether to suppress irrelevant ep info in EPD and FEN output. */
+    Boolean suppress_redundant_ep_info;
     /* Maximum depth to which to search for positional variations.
      * This is picked up from the length of variations in the positional
      * variations file.
@@ -235,13 +250,19 @@ typedef struct {
     /* Lower and upper bounds for moves if check_move_bounds. */
     unsigned lower_move_bound, upper_move_bound;
     /* Limit to the number of plies to appear in the output. */
-    unsigned output_ply_limit;
+    int output_ply_limit;
+    /* Which single game to output (matching_game_number > 0) */
+    unsigned long matching_game_number;
     /* Whether to output a FEN string at the end of the game. */
     Boolean output_FEN_string;
     /* Whether to add a FEN comment after every move. */
     Boolean add_FEN_comments;
     /* Whether to add a 'matching position' comment. */
     Boolean add_position_match_comments;
+    /* Whether to include a tag with the total ply count of the game. */
+    Boolean output_total_plycount;
+    /* Whether to add a HashCode tag. */
+    Boolean add_hashcode_tag;
     /* The comment to use for position matches, if required. */
     const char *position_match_comment;
     /* Current input file name. */
@@ -266,4 +287,4 @@ typedef struct {
  * through command line arguments.
  */
 extern StateInfo GlobalState;
-FILE *MustOpen(const char *filename,const char *mode);
+FILE *must_open_file(const char *filename,const char *mode);
